-- firewall6-ruleCLI.lua
-- Samer Sayeed
-- TeamF1
-- 10/10/08
-- 
-- Modification History
-- 27Jun12,sig Fixed SPR 32986
-- 19dec11,akr fixed 28518 
-- 19dec11,akr added qos priority in show ipv6 firewall command
-- 21nov,akr  fixed the show firewall rules
-- 17feb09,ss   added validation checks to fix spr#10808
-- 31jan09,ss  modified firewallRule6InfoGet() routine to make output more readable 
-- 10dec09,ss  written
--
-- Description
-- CLISH Firewall rule LUA routines
--
-- Required Libraries

require "teamf1lualib/fwView"
require "teamf1lualib/fwRules"

-- This function displays the current firewall configuration
function firewallRule6InfoGet (args)
    local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    if (ipMode == "1") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6 to configure this option.\n")
	return false
    end
    local firewallTable = db.getTable ("FirewallRules6")
    local resultTab = {} -- table to store firewall rules
    local firewallRow = nil
    local rowId = "0"

    printLabel ("Default Outbound Policy")
    defRow = db.getRow("FirewallRules6", "RuleType", "DEFAULT_POLICY")
    if (defRow["FirewallRules6.Action"] == "ACCEPT") then 
        print ("For IPv6 : Allow Always\n")
    elseif(defRow["FirewallRules6.Action"] == "DROP") then
        print ("For IPv6 : Block Always")
    end
    printLabel ("List of Available IPv6 Firewall Rules")    
    for col, val in pairs(firewallTable) do
        rowId = rowId+1
        local firewallRow = firewallTable[rowId]
	    if (firewallRow ~= nil) then
            local status = nil
            local ruleType = nil
            local destusrs = nil
            local srcusrs = nil
            local action = nil
            if (firewallRow["FirewallRules6._metadata"] ~= "edit=0" and firewallRow["FirewallRules6.RuleType"] ~= "DEFAULT_POLICY") then
                if(firewallRow["FirewallRules6.Status"] == "0") then
                    status =  "Disabled"
                elseif(firewallRow["FirewallRules6.Status"] == "1") then
                    status =  "Enabled"
                end
                if(firewallRow["FirewallRules6.RuleType"] == "SECURE_INSECURE") then
                    ruleType = "LAN To WAN"
                elseif(firewallRow["FirewallRules6.RuleType"] == "INSECURE_SECURE") then
                    ruleType = "WAN To LAN"
                elseif(firewallRow["FirewallRules6.RuleType"] == "INSECURE_PUBLIC") then
                    ruleType = "WAN To DMZ"
                elseif(firewallRow["FirewallRules6.RuleType"] == "PUBLIC_INSECURE") then
                    ruleType = "DMZ To WAN"
                elseif(firewallRow["FirewallRules6.RuleType"] == "PUBLIC_SECURE") then
                    ruleType = "DMZ To LAN"
                elseif(firewallRow["FirewallRules6.RuleType"] == "SECURE_PUBLIC") then
                    ruleType = "LAN To DMZ"
                end
                if (firewallRow["FirewallRules6.Action"] == "ACCEPT") then
                    action = "ALLOW Always"
                elseif (firewallRow["FirewallRules6.Action"] == "DROP") then
                    action = "BLOCK Always"
                elseif (firewallRow["FirewallRules6.Action"] == "DROP_BY_SCHEDULE") then
                    action = "BLOCK by schedule, otherwise allow"
                elseif (firewallRow["FirewallRules6.Action"] == "ACCEPT_BY_SCHEDULE") then
                    action = "ALLOW by schedule,otherwise block"
                end
                if(firewallRow["FirewallRules6.SourceAddressType"] == "0") then
                    srcusrs = "Any"
                elseif(firewallRow["FirewallRules6.SourceAddressType"] == "1") then
                    srcusrs = firewallRow["FirewallRules6.SourceAddressStart"]
                elseif(firewallRow["FirewallRules6.SourceAddressType"] == "2") then
                    srcusrs = firewallRow["FirewallRules6.SourceAddressStart"] .. " - " .. firewallRow["FirewallRules6.SourceAddressEnd"]
                end
                if(firewallRow["FirewallRules6.DestinationAddressType"] == "0") then
                    destusrs = "Any"
                elseif(firewallRow["FirewallRules6.DestinationAddressType"] == "1") then
                    destusrs = firewallRow["FirewallRules6.DestinationAddressStart"]
                elseif(firewallRow["FirewallRules6.DestinationAddressType"] == "2") then
                    destusrs = firewallRow["FirewallRules6.DestinationAddressStart"] .. " - " .. firewallRow["FirewallRules6.DestinationAddressEnd"]
                end
                if (firewallRow["FirewallRules6.LogLevel"] == "1") then
                    logLevel = "Never" 
                elseif (firewallRow["FirewallRules6.LogLevel"] == "2") then
                    logLevel = "Always"
                end 
                resTab.insertField (resultTab, "ROW ID", firewallRow["FirewallRules6._ROWID_"] or "")
                resTab.insertField (resultTab, "Status",  status or "")
                resTab.insertField (resultTab, "Rule Type",  ruleType or "")
                resTab.insertField (resultTab, "Service", firewallRow["FirewallRules6.ServiceName"] or "")
                resTab.insertField (resultTab, "Action", action or "")
                resTab.insertField (resultTab, "Source Users", srcusrs or "")
                resTab.insertField (resultTab, "Destination Users", destusrs or "")
                resTab.insertField (resultTab, "Log",  logLevel or "")
		local serviceRow = db.getRow ("Services", "ServiceName", firewallRow["FirewallRules6.ServiceName"])
		local qosPriority=""
		if(serviceRow ~= nil and serviceRow["Services.IsDefault"] == "1" and firewallRow["FirewallRules6.TypeOfService"] == nil) then
		    firewallRow["FirewallRules6.TypeOfService"] = serviceRow["Services.TypeOfService"]
		end
		if( firewallRow["FirewallRules6.TypeOfService"] == nil or firewallRow["FirewallRules6.TypeOfService"] =="" ) then
			qosPriority="Normal-Service"
		elseif( firewallRow["FirewallRules6.TypeOfService"] == "0" ) then
			qosPriority="Normal-Service"
		elseif( firewallRow["FirewallRules6.TypeOfService"] == "8" ) then
			qosPriority="Minimize-Delay"
		elseif( firewallRow["FirewallRules6.TypeOfService"] == "1" ) then
			qosPriority="Minimize-Cost"
		elseif( firewallRow["FirewallRules6.TypeOfService"] == "2" ) then
			qosPriority="Maximize-Reliability"
		elseif( firewallRow["FirewallRules6.TypeOfService"] == "4" ) then
			qosPriority="Maximize-Throughput"
		else
			qosPriority="Normal-Service"
		end
                resTab.insertField (resultTab, "Qos Priority", qosPriority or "")
		if ( firewallRow["FirewallRules6.Action"] == "ACCEPT_BY_SCHEDULE" or firewallRow["FirewallRules6.Action"] == "DROP_BY_SCHEDULE") then
		    if (firewallRow["FirewallRules6.ScheduleName"] ~= nil or firewallRow["FirewallRules6.ScheduleName"] ~= "") then 
			    resTab.insertField(resultTab,"Schedule",firewallRow["FirewallRules6.ScheduleName"] or '')
		    end
		 else 
			    resTab.insertField(resultTab,"Schedule","")
		 end
            end -- if _metadata
        end -- if rowid nil
    end -- for 
    resTab.print (resultTab, 0)
end

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function firewallSettings6CfgInit(args)
    local action = args[1]
    local configRow={}
    
    configRow["FirewallRules6.Action"] = db.getAttribute("FirewallRules6", "RuleType", "DEFAULT_POLICY", "Action")
    if(action == "1") then
      configRow["FirewallRules6.Action"] = "ACCEPT"
    else
      configRow["FirewallRules6.Action"] = "DROP"
    end
    return configRow["FirewallRules6._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function firewallSettings6CfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
	
    local rowid = db.getAttribute("FirewallRules6", "RuleType", "DEFAULT_POLICY", "_ROWID_")
    if (rowid ~= nil) then
    DBTable = "FirewallRules6"
    errorFlag, statusCode = fwView.defaultPolicyConfig(configRow, rowid, "edit")
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    else
    errorFlag = "ERROR"
    statusMessage = "Default outbound rule not found for IPv6 Firewall Rules."
    end
	
    -- save db if no error
    if (errorFlag == "OK") then db.save() end    
    return errorFlag, statusMessage
end

-- The routine checks the ip mode of the box
function firewallSettings6InVal()
    local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    if (ipMode == "1") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6 to configure this option.\n")
	return false
    end
    return true
end    
function DeleteStaleInput()
	__CLISH_CUR_CONFIG_ROW = {} -- current config info is now discarded
 	__CLISH_EDIT_CONFIG_ROW = {} -- current edit config info is discarded
	return true
end
-- This function enables/disables/deletes one rule at a time.
function accessControlRule6Modify(args)
    local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    if (ipMode == "1") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6 to configure this option.\n")
	return false
    end

    local errorFlag = "Error"
    local statusCode =""
    local statusMessage =""
    DBTable= "FirewallRules6"     

    local rowIds={}
    rowIds["rowId"] = args[1]
    local operation = args[2]
    local defRow= db.getAttribute("FirewallRules6","RuleType","DEFAULT_POLICY","_ROWID_")
    local existRow= db.getRow("FirewallRules6","_ROWID_",args[1])
    if(defRow >= args[1] and operation == "delete") then
        printCLIError("Firewall Rule cannot be deleted")
        return errorFlag, statusMessage
    elseif(existRow == nil) then
        printCLIError("Firewall Rule not exist")
        return errorFlag, statusMessage
    else
        errorFlag, statusMessage = fwView.modifyRules6(rowIds, operation)
    end
    if (errorFlag == "OK") then
        db.save()
    end
    firewallRule6InfoGet()
    return errorFlag, statusMessage
end    

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or will just initialize the defaults in case a new
-- rule is getting added.
function accessControlRule6CfgInit(args)
    local rowId = args[1]
    local configRow={}
    if (rowId == nil) then
        configRow["FirewallRules6._ROWID_"] = "-1"
        configRow["FirewallRules6.ServiceName"] = "ANY"
        configRow["FirewallRules6.SourceAddressType"] = "0"
        configRow["FirewallRules6.DestinationAddressType"] = "0"
        configRow["FirewallRules6.LogLevel"] = "1"
        configRow["FirewallRules6.Action"] ="DROP"
    else
        configRow= db.getRow("FirewallRules6","_ROWID_",rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1
        end
    end
    --configRow["FirewallRules6.Call"] = "1"
    return configRow["FirewallRules6._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function accessControlRule6CfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "FirewallRules6"

    if (configRow["FirewallRules6._ROWID_"] == "-1") then
--        if (configRow["FirewallRules6.RuleType"] == "INSECURE_SECURE") then 
--        configRow["FirewallRules6.FromZoneType"] = "INSECURE"
--        configRow["FirewallRules6.ToZoneType"] = "SECURE"
--        elseif (configRow["FirewallRules6.RuleType"] == "SECURE_INSECURE") then 
--        configRow["FirewallRules6.FromZoneType"] = "SECURE"
--        configRow["FirewallRules6.ToZoneType"] = "INSECURE"
--        end 
    	if (configRow["FirewallRules6.TypeOfService"] == nil or configRow["FirewallRules6.DestinationAddressType"] =="") then
	        configRow["FirewallRules6.TypeOfService"] = "0"  --Normal-Service
	end
        errorFlag, statusMessage = fwView.firewallRules6Config(configRow, "-1", "add")
    else
        errorFlag, statusMessage = fwView.firewallRules6Config(configRow, configRow["FirewallRules6._ROWID_"], "edit")
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "1" or configRow["FirewallRules6.SourceAddressType"] =="2") then
        configRow["FirewallRules6.SourceAddressPrefixLength"] = "128" 
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "1" or configRow["FirewallRules6.DestinationAddressType"] =="2") then
        configRow["FirewallRules6.DestinationAddressPrefixLength"] = "128" 
    end
    if (errorFlag == "OK") then
        db.save()
	DeleteStaleInput()
    end
    return errorFlag, statusMessage
end   

function accessControlRule6CfgInputVal(configRow)
    local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    if (ipMode == "1") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6 to configure this option.\n")
    	return false
    end

    if ((configRow["FirewallRules6.ServiceName"] == "nil") or (configRow["FirewallRules6.ServiceName"] == "")) then 
        printCLIError("please enter the service name")
        return false
    elseif (db.existsRowWhere("Services","ServiceName='"..configRow["FirewallRules6.ServiceName"].."' AND IsDefault=0 AND Protocol=1")) then
		printCLIError("IPv4 Custom services can not be used in IPv6 Firewall Rules !!")
		return false
    end  

    if ( configRow["FirewallRules6.Action"] == nil or configRow["FirewallRules6.Action"] == "" ) then
	     printCLIError("Please Enter Action !!")
	     return false
    end  
    if ( configRow["FirewallRules6.Action"] == "ACCEPT_BY_SCHEDULE" or configRow["FirewallRules6.Action"] == "DROP_BY_SCHEDULE") then
	    if (configRow["FirewallRules6.ScheduleName"] == nil or configRow["FirewallRules6.ScheduleName"] == "") then 
		    printCLIError("Please select Schedule !!")
		    return false
	    end
    end
    if (configRow["FirewallRules6.ServiceName"]) then 
        local where = "ServiceName = '" .. configRow["FirewallRules6.ServiceName"] .. "'"
        local rowidVal = db.existsRowWhere ("Services", where)
        
        if (rowidVal == false) then
            printCLIError("Invalid service name")
            return false
        end
    end

    if (configRow["FirewallRules6.FromZoneType"] == nil or 
        configRow["FirewallRules6.FromZoneType"] == "") then
        printCLIError ("Enter valid From Zone Type\n")
        return false
    end

    if (configRow["FirewallRules6.ToZoneType"] == nil or 
        configRow["FirewallRules6.ToZoneType"] == "") then
        printCLIError ("Enter valid To Zone Type\n")
        return false
    end

    if (configRow["FirewallRules6.SourceAddressType"] == "" or 
        configRow["FirewallRules6.SourceAddressType"] == nil) then 
        printCLIError("Enter valid source address type\n")
        return false
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "" or 
        configRow["FirewallRules6.DestinationAddressType"] == nil) then 
        printCLIError("Enter valid destination address type\n")
        return false
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "1") then
        if (configRow["FirewallRules6.SourceAddressStart"] == "" or 
         configRow["FirewallRules6.SourceAddressStart"] == nil) then 
        printCLIError("Enter valid source address\n")
        return false
        end
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "2") then
        if ((configRow["FirewallRules6.SourceAddressStart"] == "" or 
             configRow["FirewallRules6.SourceAddressStart"] == nil) or 
            (configRow["FirewallRules6.SourceAddressEnd"] == "" or 
             configRow["FirewallRules6.SourceAddressEnd"] == nil)) then
            printCLIError("Enter valid source address range\n")
            return false
        end
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "1") then 
        if (configRow["FirewallRules6.DestinationAddressStart"] == "" or 
         configRow["FirewallRules6.DestinationAddressStart"] == nil) then
       	printCLIError("Enter valid destination address\n")
       	return false
        end
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "2") then
        if((configRow["FirewallRules6.DestinationAddressStart"] == "" or 
          configRow["FirewallRules6.DestinationAddressStart"] == nil) or 
          (configRow["FirewallRules6.DestinationAddressEnd"] == "" or 
          configRow["FirewallRules6.DestinationAddressEnd"] == nil)) then
            printCLIError("Enter valid destination address range\n")
            return false
        end
    end
    return true
end
