-- ifDevCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 18apr12,sig added prefix Delegation option in net lan ipv6 configure
-- 15mar12,sig fixed for 30594
-- 09Mar12,akr fixed 30410 
-- 06dec11,akr fixed show lan dhcp configuration in CLI
-- 01dec11,akr fixed 28179
-- 18Nov11,akr fixed SPR 27911
-- 04mar10,db  modified pool related routines to fix issues.
-- 17feb09,ss  added check in lanCfgInputVal() to fix spr#10746, 
-- 23jan09,ss  added ipv6 lan setup routines, 
--             modified ipv4 lan setup, display routines. 
-- 25dec07,gnm Modified bdgCLIStatsGet() to use ifStatsPrint() to
--             print fields to CLI; Added ifStatsPrint(); Added
--	       ifCLIStatsGet() as generic interface stats print
--	       routine.
-- 23dec07,gnm modified aliasipGet(); added lanSetupGet; added
--             bdgCLIStatsGet()
-- 09dev07 gnm written
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/ifDev"
require "teamf1lualib/lan"
require "teamf1lualib/dhcp"
require "teamf1lualib/igmp"
require "teamf1lualib/validations"
require "teamf1lualib/lanGroups"
require "teamf1lualib/multiLan"
--
-- This routine validates the LAN configuration information.
function lanCfgInputVal(configRow)
    if (tonumber(configRow["vlan.vlanId"]) > 4089 or tonumber(configRow["vlan.vlanId"]) < 1 ) then
        printCLIError ("Invalid Vlan ID, enter value between 1 - 4089")
        return false
    end
    if (configRow["vlan.vlanName"] == "" or configRow["vlan.vlanName"] == nil ) then
        printCLIError ("Invalid Vlan Profile Name")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "" or configRow["dhcp.dhcpEnabled"] == nil ) then
        printCLIError ("Invalid Dhcp mode")
        return false
    end
    if (configRow["dhcp.enableDnsProxy"] == "" or configRow["dhcp.enableDnsProxy"] == nil ) then
        printCLIError ("Invalid DNS proxy status")
        return false
    end
    if (configRow["interVLANRouting.status"] == "" or configRow["interVLANRouting.status"] == nil ) then
        printCLIError ("Invalid Vlan Profile Name")
        return false
    end
    if (string.len(configRow["vlan.vlanName"]) > 16 ) then
        printCLIError ("Invalid Vlan Profile Name, Enter a name of length <= 16")
        return false
    end
    if (configRow["ifStatic.StaticIp"] == nil or 
        configRow["ifStatic.StaticIp"] == "" or 
        configRow["ifStatic.StaticIp"] == "0.0.0.0") then
        printCLIError ("Invalid IP address")
        return false
    end
    if (configRow["ifStatic.NetMask"] == nil or 
        configRow["ifStatic.NetMask"] == "" or 
        configRow["ifStatic.NetMask"] == "0.0.0.0") then
        printCLIError ("Invalid Subnet Mask")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and 
        (configRow["dhcp.domainName"] == nil or 
         configRow["dhcp.domainName"] == "")) then 
        printCLIError ("Invalid dhcp domain name")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and 
        (configRow["dhcp.startAddress"] == nil or 
         configRow["dhcp.startAddress"] == "" or 
         configRow["dhcp.startAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid Start IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and 
        (configRow["dhcp.endAddress"] == nil or 
         configRow["dhcp.endAddress"] == "" or 
         configRow["dhcp.endAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid End IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and 
        (configRow["dhcp.leaseTime"] == "" or
         configRow["dhcp.leaseTime"] == nil)) then
        printCLIError ("Invalid lease Time")
        return false
    end

    if (configRow["dhcp.dhcpEnabled"] == "1" and 
       (configRow["dhcp.RelayGateway"] == nil or 
        configRow["dhcp.RelayGateway"] == "" or 
        configRow["dhcp.RelayGateway"] == "0.0.0.0")) then
        printCLIError ("Invalid relay gateway")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == nil or
        configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == "" or
        configRow["DhcpLdapServerInfo.DhcpLdapServerIP"] == "0.0.0.0")) then
        printCLIError ("Invalid Ldap server IP")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] == nil or
        configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] == "" )) then
        printCLIError ("Invalid Ldap search base")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "2" and
        configRow["DhcpLdapServerInfo.DhcpLdapStatus"] == "1" and
       (configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] == nil or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] > "65535" or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] < "0" or
        configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] == "")) then
        printCLIError ("Invalid Ldap port")
        return false
    end
    return true
end

--
-- This routine validates the LAN configuration information.
function lanIPv6CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 LAN Configurations.")
    return false
    end
    if(configRow["dhcpv6s.isEnabled"]== "1" and configRow["dhcpv6s.useDNSServersFrom"]=="3" ) then
        if (configRow["dhcpv6s.primaryDNSServer"] == nil or configRow["dhcpv6s.primaryDNSServer"] == "" ) then
            printCLIError("Please enter a valid Primary DNS Server.")
            return false
        end
    end
    if ((configRow["ifStatic.PrefixLength"] == nil) or (tonumber(configRow["ifStatic.PrefixLength"]) < 0) or (tonumber(configRow["ifStatic.PrefixLength"]) > 128 ))then
        printCLIError ("Invalid Prefix. Please enter a value between 0 and 128.")
        return false
    end

    return true
end

--
-- This routine validates the LAN configuration information.
function lanIPv6CfgPoolInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 LAN Configurations.")
    return false
    end
    if (configRow ["dhcpv6sLANAddrPool.startAddress"] == "" or configRow ["dhcpv6sLANAddrPool.startAddress"] == nil) then
         printCLIError("Please Enter valid IPv6 Start Address.")
         return false

    end
    if (configRow ["dhcpv6sLANAddrPool.endAddress"] == "" or configRow ["dhcpv6sLANAddrPool.endAddress"] == nil) then
         printCLIError("Please Enter valid IPv6 End Address.")
         return false
    end
    if (configRow["dhcpv6sLANAddrPool.prefixLength"] == "" or configRow["dhcpv6sLANAddrPool.prefixLength"] == nil)then
        printCLIError ("Please Enter valid Prefix Length.")
        return false
    end

    return true
end

--
-- This routine saves the modified LAN configuration information.
function lanCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local ifstaticRows = ''
    if (configRow["vlan.vlanId"] ~= "1") then
        ifstaticRows = db.getRowsWhere("ifStatic","AddressFamily = 2 and LogicalIfName!='LAN"..configRow["vlan.vlanId"].."'")
    else
        ifstaticRows = db.getRowsWhere("ifStatic","AddressFamily = 2 and LogicalIfName!='LAN'")
    end
    local j = 0
    local dmzPort = db.getAttribute("ConfigPort","_ROWID_","1","LogicalIfName")
    if (#ifstaticRows ~= 0) then
        for k,v in pairs(ifstaticRows) do
            j = j + 1
            local ifstatic = ifstaticRows[j]
            if (ifstatic["ifStatic.LogicalIfName"] == "WAN1") then
                local ipAddrRow = db.getRowWhere("ipAddressTable","addressFamily = 2 and LogicalIfName='WAN1'")
                if (ipAddrRow ~= nil) then
                    ifstatic["ifStatic.StaticIp"] = ipAddrRow["ipAddressTable.ipAddress"] or ""
                    ifstatic["ifStatic.NetMask"] = ipAddrRow["ipAddressTable.subnetMask"] or ""
                end
            end
            if (ifstatic ~= nil and ifstatic["ifStatic.StaticIp"] ~= nil and
                ifstatic["ifStatic.StaticIp"] ~= "0.0.0.0" and ifstatic["ifStatic.StaticIp"] ~= "") then
		        if((ifstatic["ifStatic.LogicalIfName"] ~= "DMZ") or (dmzPort == "DMZ" and ifstatic["ifStatic.LogicalIfName"] == "DMZ")) then
				    errorFlag = ifDevLib.ipSubnet(ifstatic["ifStatic.StaticIp"],ifstatic["ifStatic.NetMask"],configRow["ifStatic.StaticIp"])
				    local var ="Error: Invalid Start IP Address. " .. configRow["ifStatic.StaticIp"] .." is out of subnet range"
				    if(errorFlag ~= var) then
            		    printCLIError ("IP address conflicting with "..ifstatic["ifStatic.LogicalIfName"])
                  	   	return false
                    end
		  		end
            end
        end
    end
		local where = "LogicalIfName = 'LAN' AND AddressFamily = 2"
		local rowid = db.existsRowWhere ("ifStatic", where)
        local RowId = "-1"
        local row = db.getRowWhere ("vlan","portName='dummy' and vlanId='" .. configRow["vlan.vlanId"] .."'")
        if (row ~= nil) then
        RowId = row["vlan._ROWID_"]
        if ( configRow ["vlan.vlanName"] ~= row ["vlan.vlanName"]) then
            configRow ["vlan.vlanName"] = row ["vlan.vlanName"]
            print("can't change Vlan Profile name")
        end
        end
        configRow ["vlan.vlanName2"] = configRow ["vlan.vlanName"]
        configRow ["vlan.portName"] = "dummy"
        configRow ["vlan.portName2"] = "port10"
        local i = 0
        if (configRow ["ifStatic.StaticIp"] ~= nil) then
            local a = util.split (configRow ["ifStatic.StaticIp"],".")
            for k,v in pairs (a) do
                i = i + 1
                configRow ["ifStatic.IPByte"..i] = a[i]
            end
        end
        i = 0
        if (configRow ["ifStatic.NetMask"] ~= nil) then
            local b = util.split (configRow ["ifStatic.NetMask"],".")
            for k,v in pairs (b) do
                i = i + 1
                configRow ["ifStatic.SMByte"..i] = b[i]
            end
        end
        i = 0
        if (configRow ["dhcp.startAddress"] ~= nil) then
            local c = util.split (configRow ["dhcp.startAddress"],".")
            for k,v in pairs (c) do
                i = i + 1
                configRow ["dhcp.startAddress"..i] = c[i]
            end
        end
        i = 0
        if (configRow ["dhcp.endAddress"] ~= nil) then
            local d = util.split (configRow ["dhcp.endAddress"],".")
            for k,v in pairs (d) do
                i = i + 1
                configRow ["dhcp.endAddress"..i] = d[i]
            end
        end
        i = 0
        if (configRow ["dhcp.primaryDnsServer"] ~= nil) then
            local e = util.split (configRow ["dhcp.primaryDnsServer"],".")
            for k,v in pairs (e) do
                i = i + 1
                configRow ["dhcp.primaryDnsServer"..i] = e[i]
            end
        end
        i = 0
        if (configRow ["dhcp.secondaryDnsServer"] ~= nil) then
            local f = util.split (configRow ["dhcp.secondaryDnsServer"],".")
            for k,v in pairs (f) do
                i = i + 1
                configRow ["dhcp.secondaryDnsServer"..i] = f[i]
            end
        end
        i = 0
        if (configRow ["dhcp.winsServer"] ~= nil) then
            local g = util.split (configRow ["dhcp.winsServer"],".")
            for k,v in pairs (g) do
                i = i + 1
                configRow ["dhcp.winsServer"..i] = g[i]
            end
        end
        i = 0
        if (configRow ["dhcp.RelayGateway"] ~= nil) then
            local h = util.split (configRow ["dhcp.RelayGateway"],".")
            for k,v in pairs (h) do
                i = i + 1
                configRow ["dhcp.RelayGateway"..i] = h[i]
            end
        end
        local j = 0
        for k = 1,4 do
            j = j + 1
            if (configRow ["vlan.port"..j] ~= "1") then
                configRow ["vlan.port"..j] = "0"
            end
        end
        if (RowId == "-1") then
            errorFlag, statusCode = vlanConfig.vlanAddWr (configRow, RowId, "add")
        else
            errorFlag, statusCode = vlanConfig.vlanAddWr (configRow, RowId, "edit")
        end
        if (errorFlag == "OK") then
		    db.save()
		end
	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    if (errorFlag ~= "OK") then
        print(statusMessage)
    end
        return errorFlag, statusMessage
end

--
-- This routine saves the modified LAN configuration information.
function lanIPv6CfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    if(configRow["dhcpv6s.isEnabled"]=="0" )then
	if( configRow["dhcpv6s.prefixDelegation"]=="1") then
		configRow["dhcpv6s.prefixDelegation"]= "0"
        end
    end	
    if(configRow["dhcpv6s.isEnabled"]=="1" and configRow["dhcpv6s.statelessMode"]=="0")  then
	if(configRow["dhcpv6s.prefixDelegation"] == "1") then
    		printCLIError("Invalid prefix Delegation.Prefix Delegation is set only for stateless mode.")
		return false
	end
    end
    if (configRow["dhcpv6s.prefixDelegation"] == "" or configRow["dhcpv6s.prefixDelegation"] == nil) then
        configRow["dhcpv6s.prefixDelegation"] = "0"
    end

    local where = "LogicalIfName = 'LAN' AND AddressFamily = '10'"
	local rowid = db.existsRowWhere ("ifStatic", where)
	local valid = lan.obsoleteipv6LanConfig(configRow, rowid, "edit")

	-- save db if no error
    if (valid) then 
        statusCode = "STATUS_OK"
	errorFlag = "OK"
        db.save()
    else
	errorFlag = "ERROR"
	statusCode = "LAN_CONFIG_FAILED"
    end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage           	 
end

--
-- This routine reads in the current IPv6 LAN configuration information
-- into a configuration object.
function lanIPv6CfgInit(args)
    configRow = db.getRow("dhcpv6s", "_ROWID_", "1")
	if (configRow == nil) then
	    configRow = db.getDefaults(true, "dhcpv6s")
	end
    local staticRow = {}
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 10"
    local rowid = db.existsRowWhere ("ifStatic", where)
    staticRow = db.getRow("ifStatic", "_ROWID_", rowid )
    if(staticRow ~= nil) then
        configRow["ifStatic.StaticIp"] = staticRow["ifStatic.StaticIp"]
        configRow["ifStatic.PrefixLength"] = staticRow["ifStatic.PrefixLength"]
    end
    return rowid, configRow
end

--
-- This routine reads in the current IPv4 LAN configuration information
-- into a configuration object.
function lanCfgInit(args)
    local where = ''
    if (args[1] ~= "1") then
        where = "LogicalIfName = 'LAN"..args[1].."' AND AddressFamily = 2"
    else
        where = "LogicalIfName = 'LAN' AND AddressFamily = 2"
    end
	local ifstaticRow = db.getRowWhere ("ifStatic", where)
    local configRow = {}
    if (ifstaticRow ~= nil) then
        configRow ["ifStatic.StaticIp"] = ifstaticRow ["ifStatic.StaticIp"]
        configRow ["ifStatic.NetMask"] = ifstaticRow ["ifStatic.NetMask"]
    end
    configRow ["dhcp.enableDnsProxy"] = "0"
    local dhcpConfigRow = ''
    if (args[1] ~= "1") then
        dhcpConfigRow = db.getRow ("dhcp", "logicalIfName", "LAN"..args[1])
    else
        dhcpConfigRow = db.getRow ("dhcp", "logicalIfName", "LAN")
    end
    if (dhcpConfigRow ~= nil) then
    	configRow ["dhcp.dhcpEnabled"] = dhcpConfigRow["dhcp.dhcpEnabled"]
   		if (configRow ["dhcp.dhcpEnabled"] ~= "0") then
    		configRow ["dhcp.domainName"] = dhcpConfigRow ["dhcp.domainName"]
    		configRow ["dhcp.startAddress"] = dhcpConfigRow ["dhcp.startAddress"]
            configRow ["dhcp.endAddress"] = dhcpConfigRow ["dhcp.endAddress"]
            configRow ["dhcp.primaryDnsServer"] = dhcpConfigRow ["dhcp.primaryDnsServer"] or "" 
            configRow ["dhcp.secondaryDnsServer"] = dhcpConfigRow ["dhcp.secondaryDnsServer"] or "" 
            configRow ["dhcp.winsServer"] = dhcpConfigRow ["dhcp.winsServer"] or ""
            configRow ["dhcp.leaseTime"] = dhcpConfigRow ["dhcp.leaseTime"]
            configRow ["dhcp.enableDnsProxy"] = dhcpConfigRow ["dhcp.enableDnsProxy"]
            configRow ["dhcp.dhcpEnabled"] = "2"
		elseif(configRow["dhcp.dhcpEnabled"] == "0" and args[1] ~= nil and args[1] ~= "")then
		    if(args[1] ~= '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN"..args[1],"DhcpRelayStatus") == "1") then
			    configRow["dhcp.dhcpEnabled"] = "1" --dhcp-relay 
			    configRow["dhcp.RelayGateway"] = db.getAttribute("DhcpRelay","LogicalIfName","LAN"..args[1],"RelayGateway") or ''  --dhcp-relay-server ip
		    elseif(args[1] == '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN","DhcpRelayStatus") == "1") then
			    configRow["dhcp.dhcpEnabled"] = "1" --dhcp-relay 
			    configRow["dhcp.RelayGateway"] = db.getAttribute("DhcpRelay","LogicalIfName","LAN","RelayGateway") or ''  --dhcp-relay-server ip
		   else
			    configRow["dhcp.dhcpEnabled"] = "0" --dhcp-NONE 
		  
	    	   end
    	end
    end
    local i = 0
    configRow ["vlan.vlanId"] = args[1]
    local vlantable = db.getRowsWhere ("vlan","vlanId="..args[1])
    if (#vlantable ~= 0) then
        configRow ["vlan.vlanName"] = vlantable[1] ["vlan.vlanName"]
        for k,v in pairs(vlantable) do
            i = i + 1
            local row = vlantable[i]
            if (row ~= nil) then
                if (row ["vlan.portName"] == "port1") then
                    configRow ["vlan.port1"] = "1"
                elseif (row ["vlan.portName"] == "port2") then
                    configRow ["vlan.port2"] = "1"
                elseif (row ["vlan.portName"] == "port3") then
                    configRow ["vlan.port3"] = "1"
                elseif (row ["vlan.portName"] == "port4") then
                    configRow ["vlan.port4"] = "1"
                --[[elseif (row ["vlan.portName"] == "port5") then
                    configRow ["vlan.port5"] = "1"
                elseif (row ["vlan.portName"] == "port6") then
                    configRow ["vlan.port6"] = "1"
                elseif (row ["vlan.portName"] == "port7") then
                    configRow ["vlan.port7"] = "1"
                elseif (row ["vlan.portName"] == "port8") then
                    configRow ["vlan.port8"] = "1"]]--
                end
            end
        end
    end
    configRow ["DhcpLdapServerInfo.DhcpLdapStatus"] = "0"
    local ldapconfigRow = ''
    if (args[1] ~= "1") then
        ldapconfigRow = db.getRow ("DhcpLdapServerInfo", "LogicalIfName", "LAN"..args[1])
    else
        ldapconfigRow = db.getRow ("DhcpLdapServerInfo", "LogicalIfName", "LAN")
    end
    if (ldapconfigRow ~= nil) then
        configRow ["DhcpLdapServerInfo.DhcpLdapStatus"] = ldapconfigRow ["DhcpLdapServerInfo.DhcpLdapStatus"]
        if (configRow ["DhcpLdapServerInfo.DhcpLdapStatus"] ~= 0) then
            configRow ["DhcpLdapServerInfo.DhcpLdapServerIP"] = ldapconfigRow ["DhcpLdapServerInfo.DhcpLdapServerIP"]
            configRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"] = ldapconfigRow ["DhcpLdapServerInfo.DhcpLdapSearchBase"]
            configRow ["DhcpLdapServerInfo.DhcpLdapServerPort"] = ldapconfigRow ["DhcpLdapServerInfo.DhcpLdapServerPort"]
        end
    end
    configRow ["interVLANRouting.status"] = "0"
    local intervlanroutingRow = db.getRow ("interVLANRouting","vlanId",args[1])
    if (intervlanroutingRow ~= nil) then
        configRow ["interVLANRouting.status"] = intervlanroutingRow ["interVLANRouting.status"]
    end
--[[
    local dhcpRelayConfig = ''
    if (args[1] ~= "1") then
        dhcpRelayConfig = db.getRow("DhcpRelay", "LogicalIfName", "LAN"..args[1])
    else
        dhcpRelayConfig = db.getRow("DhcpRelay", "LogicalIfName", "LAN")
    end
    if (dhcpRelayConfig ~= nil) then
        configRow ["dhcp.RelayGateway"] = dhcpRelayConfig["dhcp.RelayGateway"]
    	if (dhcpRelayConfig["dhcp.RelayGateway"] == "1") then
    		configRow ["dhcp.dhcpEnabled"] = "1"
    	    configRow ["dhcp.RelayGateway"] = dhcpRelayConfig["dhcp.RelayGateway"]
    	end
    end
  ]]
    local igmpConfig = db.getRow("Igmp", "_ROWID_", "1")
    if (igmpConfig ~=nil) then
        configRow ["Igmp.IgmpEnable"] = igmpConfig["Igmp.IgmpEnable"]
    end
    return rowid, configRow
end

function lanIPv6SetupGet (args)
    local resultTab = {}
    local resultTabPool = {}
    local dhcpv6sRow = db.getRow("dhcpv6s", "_ROWID_", "1")
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 10"
    local rowid = db.existsRowWhere ("ifStatic", where)
    local ipv6LanRow = db.getRow("ifStatic", "_ROWID_", rowid )
    local dhcpv6sPoolTable = db.getTable ("dhcpv6sLANAddrPool")
    local dhcpv6sPoolRow = nil
    local rowId = 0

    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to access IPv6 LAN Configurations.")
    return false
    end

    printLabel ("IPv6 LAN Configuration")
    resTab.insertField (resultTab, "LAN TCP/IP Setup","")
    if(ipv6LanRow ~= nil) then
        resTab.insertField (resultTab, "IPv6 Address", ipv6LanRow["ifStatic.StaticIp"] or "")
        resTab.insertField (resultTab, "IPv6 Prefix Length", ipv6LanRow["ifStatic.PrefixLength"]or "")   
    end

    resTab.insertField (resultTab, "DHCPv6", "")
    if(dhcpv6sRow ~= nil) then
       if(dhcpv6sRow["dhcpv6s.isEnabled"]=="1")then
           resTab.insertField (resultTab, "DHCP Status", "Enable DHCPv6 Server")
       else
           resTab.insertField (resultTab, "DHCP Status", "Disable DHCPv6 Server")
       end
       if(dhcpv6sRow["dhcpv6s.statelessMode"]=="1")then
           resTab.insertField (resultTab, "DHCP Mode", "Stateless")
       else
           resTab.insertField (resultTab, "DHCP Mode", "Stateful")
       end
       if(dhcpv6sRow["dhcpv6s.prefixDelegation"]=="1")then
           resTab.insertField (resultTab, "Prefix Delegation", "Enable")
       else
           resTab.insertField (resultTab, "Prefix Delegation", "Disable")
       end
       resTab.insertField (resultTab, "Domain Name", dhcpv6sRow["dhcpv6s.domainName"]or "")
       resTab.insertField (resultTab, "Server Preference", dhcpv6sRow["dhcpv6s.serverPreference"]or "")
       if(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="1")then
           resTab.insertField (resultTab, "DNS Servers", "Use DNS Proxy")
       elseif(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="2")then
           resTab.insertField (resultTab, "DNS Servers", "Use DNS from ISP")
       elseif(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="3")then
           resTab.insertField (resultTab, "DNS Servers", "Use Below")
           resTab.insertField (resultTab, "Primary DNS Server", dhcpv6sRow["dhcpv6s.primaryDNSServer"]or "")
           resTab.insertField (resultTab, "Secondary DNS Server", dhcpv6sRow["dhcpv6s.secondaryDNSServer"]or "")
       end
       resTab.insertField (resultTab, "Lease/Rebind Time", dhcpv6sRow["dhcpv6s.leaseTime"]or "")
    end
   
    for col, val in pairs(dhcpv6sPoolTable) do
        rowId = rowId+1
        local dhcpv6sPoolRow = dhcpv6sPoolTable[rowId]
        if (dhcpv6sPoolRow ~= nil) then 
            resTab.insertField (resultTabPool, "Row Id", dhcpv6sPoolRow["dhcpv6sLANAddrPool._ROWID_"] or "")
            resTab.insertField (resultTabPool, "Start Address", dhcpv6sPoolRow["dhcpv6sLANAddrPool.startAddress"] or "")
            resTab.insertField (resultTabPool, "End Address", dhcpv6sPoolRow["dhcpv6sLANAddrPool.endAddress"] or "")
            resTab.insertField (resultTabPool, "Prefix Length", dhcpv6sPoolRow["dhcpv6sLANAddrPool.prefixLength"] or "")
        end
    end
    resTab.print (resultTab, 0)
    printLabel ("List of IPv6 Address Pools")
    resTab.print (resultTabPool, 0)

    local i = 0
    local dhcpv6sPrefixTable = db.getTable ("dhcpv6sLANPrefixPool")
    local resultTabPrefix = {}
    for k, v in pairs(dhcpv6sPrefixTable) do
        i = i + 1
        local row = dhcpv6sPrefixTable[i]
        if (row ~= nil) then 
            resTab.insertField (resultTabPrefix, "Row Id", row["dhcpv6sLANPrefixPool._ROWID_"] or "")
            resTab.insertField (resultTabPrefix, "IPv6 Prefix", row["dhcpv6sLANPrefixPool.delegationPrefix"] or "")
            resTab.insertField (resultTabPrefix, "IPv6 Prefix Length", row["dhcpv6sLANPrefixPool.delegationPrefixLen"] or "")
        end
    end
    printLabel ("List of Prefixes for Prefix Delegation")
    resTab.print (resultTabPrefix, 0)
end  

--
-- This routine implements the CLI command to show the LAN
-- configuration
function lanSetupGet (args)
    local resultTab = {} -- table to store LAN setup information
    local vlanrows = db.getRowsWhere("vlan","portName='dummy' and vlanId!=4094")

    printLabel ("LAN Setup (IPv4)")
    printLabel ("VLAN Profiles")
    local i = 0
    local ifStaticRow = ''
    local dhcp = ''
    for k,v in pairs(vlanrows) do
        i = i + 1
        local vlan = vlanrows[i]
        if (vlan ~= nil) then
            if (vlan["vlan.vlanProfState"] == "1") then
                resTab.insertField (resultTab, "Status", "Enabled")
            else
                resTab.insertField (resultTab, "Status", "Disabled")
            end
            resTab.insertField (resultTab, "Profile Name", vlan["vlan.vlanName"])
            resTab.insertField (resultTab, "VLAN Id", vlan["vlan.vlanId"])
            if (vlan["vlan.vlanId"] ~= "1") then
                ifStaticRow = db.getRowWhere("ifStatic","AddressFamily = 2 AND LogicalIfName = 'LAN"..vlan["vlan.vlanId"].."'")
            else
                ifStaticRow = db.getRowWhere("ifStatic","LogicalIfName = 'LAN' AND AddressFamily = 2")
            end
                resTab.insertField (resultTab, "IPv4 Address", ifStaticRow["ifStatic.StaticIp"] or "")
                resTab.insertField (resultTab, "Subnet Mask", ifStaticRow["ifStatic.NetMask"] or "")   
            if (vlan["vlan.vlanId"] ~= "1") then
                dhcp = db.getRowWhere("dhcp","logicalIfName = 'LAN"..vlan["vlan.vlanId"].."'")
            else
                dhcp = db.getRowWhere("dhcp","logicalIfName = 'LAN'")
            end
            if (dhcp["dhcp.dhcpEnabled"] == "1") then
                resTab.insertField (resultTab, "DHCP Status", "DHCP Server")
		resTab.insertField (resultTab, "Server Address",dhcp["dhcp.startAddress"] .. " - " .. dhcp["dhcp.endAddress"])
            else
		    if(vlan["vlan.vlanId"] ~= '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN"..vlan["vlan.vlanId"],"DhcpRelayStatus") == "1") then
			    resTab.insertField(resultTab,"DHCP Status","DHCP Relay")
			    resTab.insertField (resultTab, "Server Address",db.getAttribute("DhcpRelay","LogicalIfName","LAN"..vlan["vlan.vlanId"],"RelayGateway") or '')
		    elseif(vlan["vlan.vlanId"] == '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN","DhcpRelayStatus") == "1") then
			    resTab.insertField(resultTab,"DHCP Status","DHCP Relay")
			    resTab.insertField (resultTab, "Server Address",db.getAttribute("DhcpRelay","LogicalIfName","LAN","RelayGateway") or '')
		    else
	                    resTab.insertField (resultTab, "DHCP Status", "Disabled")
		            resTab.insertField (resultTab, "Server Address","Not Applicable")
		    end
            end
        end
        vlan = nil
    end
    resTab.print (resultTab, 0)
    local resultTab1 = {}
    printLabel ("Default VLAN")
    local j = 0
    for k=1,4 do
        j = j + 1
        local vlanId = db.getAttribute("VlanPortMgmt","PortName","Port"..j,"DefaultPVID") or ''
        if (vlanId ~= "4094") then
            local vlanName = db.getAttribute("vlan","vlanId",vlanId,"vlanName")
            resTab.insertField (resultTab1, "Port"..j, vlanName)
        else
            resTab.insertField (resultTab1, "Port"..j, "DMZ")
        end
    end

    resTab.print (resultTab1, 0)
end

function lanDtldSetupGet (args)
    local resultTab = {} -- table to store LAN setup information
    if (db.existsRow("vlan","vlanId",args[1])) then
        local vlan = db.getRowWhere("vlan","portName='dummy' and vlanId="..args[1])

        if (vlan ~= nil) then
            local ifStaticRow = ''
            local dhcp = ''
            printLabel ("Detailed Setup (IPv4) of VLAN :- "..vlan["vlan.vlanName"])
            if (vlan["vlan.vlanProfState"] == "1") then
                resTab.insertField (resultTab, "Status: ", "Enabled")
            else
                resTab.insertField (resultTab, "Status: ", "Disabled")
            end
            resTab.insertField (resultTab, "Profile Name: ", vlan["vlan.vlanName"])
            resTab.insertField (resultTab, "VLAN Id: ", vlan["vlan.vlanId"])
            if (vlan["vlan.vlanId"] ~= "1") then
                ifStaticRow = db.getRowWhere("ifStatic","AddressFamily = 2 AND LogicalIfName = 'LAN"..vlan["vlan.vlanId"].."'")
            else
                ifStaticRow = db.getRowWhere("ifStatic","LogicalIfName = 'LAN' AND AddressFamily = 2")
            end
            resTab.insertField (resultTab, "IPv4 Address: ", ifStaticRow["ifStatic.StaticIp"] or "")
            resTab.insertField (resultTab, "Subnet Mask: ", ifStaticRow["ifStatic.NetMask"] or "")   
            if (vlan["vlan.vlanId"] ~= "1") then
                dhcp = db.getRowWhere("dhcp","logicalIfName = 'LAN"..vlan["vlan.vlanId"].."'")
            else
                dhcp = db.getRowWhere("dhcp","logicalIfName = 'LAN'")
            end
            if (dhcp["dhcp.dhcpEnabled"] == "1") then
                resTab.insertField (resultTab, "DHCP Status: ", "DHCP Server")
		        resTab.insertField (resultTab, "Server Address: ",dhcp["dhcp.startAddress"] .. " - " .. dhcp["dhcp.endAddress"])
                resTab.insertField (resultTab, "Primary DNS Server: ", dhcp["dhcp.primaryDnsServer"] or "")
                resTab.insertField (resultTab, "Secondary DNS Server: ", dhcp["dhcp.secondaryDnsServer"] or "")
                resTab.insertField (resultTab, "WINS Server: ", dhcp["dhcp.winsServer"] or "")
                resTab.insertField (resultTab, "Lease Time: ", dhcp["dhcp.leaseTime"])
                local ldap = ''
                if (vlan["vlan.vlanId"] ~= "1") then
                    ldap = db.getRowWhere("DhcpLdapServerInfo","LogicalIfName = 'LAN"..vlan["vlan.vlanId"].."'")
                else
                    ldap = db.getRowWhere("DhcpLdapServerInfo","LogicalIfName = 'LAN'")
                end
                if (ldap["DhcpLdapServerInfo.DhcpLdapStatus"] == "0") then
                    resTab.insertField (resultTab, "LDAP Status: ", "Disabled")
                else
                    resTab.insertField (resultTab, "LDAP Status: ", "Enabled")
                    resTab.insertField (resultTab, "LDAP Server: ", ldap["DhcpLdapServerInfo.DhcpLdapServerIP"] or "")
                    resTab.insertField (resultTab, "LDAP Search Base: ", ldap["DhcpLdapServerInfo.DhcpLdapSearchBase"] or "")
                    resTab.insertField (resultTab, "LDAP Port: ", ldap["DhcpLdapServerInfo.DhcpLdapServerPort"] or "")
                end
            else
		    if(vlan["vlan.vlanId"] ~= '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN"..vlan["vlan.vlanId"],"DhcpRelayStatus") == "1") then
			    resTab.insertField(resultTab,"DHCP Status","DHCP Relay")
			    resTab.insertField (resultTab, "Server Address",db.getAttribute("DhcpRelay","LogicalIfName","LAN"..vlan["vlan.vlanId"],"RelayGateway") or '')
		    elseif(vlan["vlan.vlanId"] == '1' and db.getAttribute("DhcpRelay","LogicalIfName","LAN","DhcpRelayStatus") == "1") then
			    resTab.insertField(resultTab,"DHCP Status","DHCP Relay")
			    resTab.insertField (resultTab, "Server Address",db.getAttribute("DhcpRelay","LogicalIfName","LAN","RelayGateway") or '')
		    else
	                    resTab.insertField (resultTab, "DHCP Status", "Disabled")
		            resTab.insertField (resultTab, "Server Address","Not Applicable")
		    end
            end
            if (dhcp["dhcp.enableDnsProxy"] == "1") then
                resTab.insertField (resultTab, "DNS Proxy: ", "Enabled")
            else
                resTab.insertField (resultTab, "DNS Proxy: ", "Disabled")
            end
            local interVlanRoutingStatus = db.getAttribute ("interVLANRouting", "vlanId", vlan["vlan.vlanId"], "status")
            if (interVlanRoutingStatus == "1") then
                resTab.insertField (resultTab, "Inter VLAN Routing: ", "Enabled")
            else
                resTab.insertField (resultTab, "Inter VLAN Routing: ", "Disabled")
            end
        end
    else
        printCLIError ("No Vlan profile present with this Vlan Id")
        return false
    end
    resTab.print (resultTab, 0)
end


--
-- This routine implements the CLI command to delete a VLAN profile
function lanprofDel(args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local vlanid = args[1]
    local rows = {}
    rows["vlanId"] = vlanid
    local vlan = db.getRowsWhere("vlan","vlanId="..vlanid)
    if (#vlan == 0) then
        printCLIError ("No VLAN profile exist with this VLAN ID")
        return false
    else
        errorFlag, statusCode = vlanConfig.vlanDeleteWr (rows)
    end
    if (errorFlag == "OK") then
        local vlanTable = db.getRowsWhere ("vlan","vlanId="..vlanid)
        if (#vlanTable ~= 0) then
            local i = 0
            for k,v in pairs(vlanTable) do
                i = i + 1
                local vlanRow = vlanTable[i] 
                errorFlag, statusCode = db.deleteRow ("vlan", "_ROWID_", vlanRow["vlan._ROWID_"])
            end
        end
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode	   
end

--
-- This routine implements the CLI command to enable a VLAN profile
function lanprofEnable(args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local vlanid = {args[1]}
    local vlan = db.getRowsWhere("vlan","portName='dummy' and vlanId="..args[1])
    if (#vlan == 0) then
        printCLIError ("No VLAN profile exist with this VLAN ID")
        return false
    else
        errorFlag, statusCode = vlanConfig.vlanModify(vlanid, "enable")
    end
    -- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode	   
end

--
-- This routine implements the CLI command to disable a VLAN profile
function lanprofDisable(args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local vlanid = {args[1]}
    local vlan = db.getRowsWhere("vlan","portName='dummy' and vlanId="..args[1])
    if (#vlan == 0) then
        printCLIError ("No VLAN profile exist with this VLAN ID")
        return false
    else
        errorFlag, statusCode = vlanConfig.vlanModify(vlanid, "disable")
    end
    -- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode	   
end

--
-- This routine reads in the current dhcpv6sLANAddrPool configuration information
-- into a configuration object.
function lanIPv6PoolInit (args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    local configRow = {}
    if(args[1] ~= nil) then
        configRow = db.getRow("dhcpv6sLANAddrPool","_ROWID_", args[1])
        if (configRow == nil) then
            printCLIError ("Row doesn't exists")
            return -1, {}
        end
    else
        configRow ["dhcpv6sLANAddrPool._ROWID_"] = "-1"
    end
    return configRow ["dhcpv6sLANAddrPool._ROWID_"], configRow
end

--
-- This routine saves the modified dhcpv6sLANAddrPool information.
function lanIPv6PoolSave (configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANAddrPool"

    if (configRow ["dhcpv6sLANAddrPool._ROWID_"] == "-1") then
        errorFlag, statusCode = lan.poolConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = lan.poolConfig(configRow, configRow ["dhcpv6sLANAddrPool._ROWID_"], "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine delete the modified dhcpv6sLANAddrPool configuration information.
function lanIPv6PoolDel (args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANAddrPool"

    local configRow = db.getRow("dhcpv6sLANAddrPool","_ROWID_", args[1])
    if (configRow == nil) then
        printCLIError ("Row doesn't exists")
        return false
    end
    row = {}
    row["_ROWID_"] = configRow ["dhcpv6sLANAddrPool._ROWID_"]
    errorFlag, statusCode = lan.lanPoolsDelete(configRow)

    -- save db if no error
    if (errorFlag == "OK") then 
        db.save() 
        print ("Row deleted")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end
  
--
-- This routine takes in a table which has sets of interface
-- statistics and prints them
function ifStatsPrint (table)
    local resultTab = {}

    for _, row in pairs(table) do
	resTab.insertField (resultTab, "IFACE", row["interfaceStats.interfaceName"])
	resTab.insertField (resultTab, "PktRx", row["interfaceStats.rx_packets"])
	resTab.insertField (resultTab, "PktTx", row["interfaceStats.tx_packets"])
	resTab.insertField (resultTab, "ByteRx", row["interfaceStats.rx_bytes"])
	resTab.insertField (resultTab, "ByteTx", row["interfaceStats.tx_bytes"])
	resTab.insertField (resultTab, "ErrRx", row["interfaceStats.rx_errors"])
	resTab.insertField (resultTab, "ErrTx", row["interfaceStats.tx_errors"])
	resTab.insertField (resultTab, "DropRx", row["interfaceStats.rx_dropped"])
	resTab.insertField (resultTab, "DropTx", row["interfaceStats.tx_dropped"])
	resTab.insertField (resultTab, "Mcast", row["interfaceStats.multicast"])
	resTab.insertField (resultTab, "Coll", row["interfaceStats.collisions"])
    end
    printLabel ("Interface Statistics")
    resTab.print (resultTab, 0)
end

--
-- This routine implements the CLI command to display bridge interface
-- statistics.
function bdgCLIStatsGet (args)
    -- run program to read the stats and update DB
    local prog = db.getAttribute("environment", "name", "IFDEVSTATS_PROGRAM", "value")
    os.execute(prog .. " " .. DB_FILE_NAME)

    local bdgTab = db.getTable ("bridgeTable")
    local resultTab = {}
    local tableIfaceInfo = {}

    for _,v in pairs(bdgTab) do
        local row = db.getRow("interfaceStats", "interfaceStats.interfaceName", 
	      v["bridgeTable.interfaceName"])
	if (row ~= nil) then
	    table.insert (tableIfaceInfo, row)
        end
    end

    ifStatsPrint (tableIfaceInfo)
    resTab.print (resultTab, 0)
end   

--
-- This routine can be used to print statistics for a set of
-- one or more interfaces. The set of interfaces is passed to this
-- routine as a table with interface names are it's elements
function ifCLIStatsGet (args)
    local tableIfaceInfo = {}	 
    local row = nil

    -- run program to read the stats and update DB
    local prog = db.getAttribute("environment", "name", "IFDEVSTATS_PROGRAM", "value")
    os.execute(prog .. " " .. DB_FILE_NAME)
    for _, iface in pairs (args) do
	row = db.getRow ("interfaceStats", "interfaceStats.interfaceName", iface)
	if (row ~= nil) then
	    table.insert (tableIfaceInfo, row)
        end
    end
    ifStatsPrint (tableIfaceInfo)
end
   

-- This routine reads in the current LAN Advanced configuration information
-- into a configuration object.
function lanAdvCfgInit(args)
    configRow = db.getRow ("vlanMac","_ROWID_","1")
    configRow ["dhcp.enableArpBcast"] = db.getAttribute ("dhcp","logicalIfName","LAN","enableArpBcast")
    return rowId, configRow
end

-- This routine validates the LAN configuration information.
function lanAdvCfgInputVal(configRow)
    return true
end

-- This routine saves the modified LAN configuration information.
function lanAdvCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    errorFlag, statusCode = vlanConfig.vlanMacConfig(configRow, "1", "edit")
	if (errorFlag == "OK") then
        db.save()
		local dhcpRowId = db.getAttribute("dhcp", "logicalIfName", "LAN", "_ROWID_")
		errorFlag, statusCode = db.setAttribute("dhcp", "_ROWID_", dhcpRowId, "enableArpBcast" , configRow["dhcp.enableArpBcast"])
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine implements the CLI command to show the LAN
-- configuration
function lanAdvSetupGet (args)
    local resultTab = {} -- table to store LAN setup information
    local mac = ""
    local arp = ""
    local LanAdvRow = db.getRow("vlanMac", "_ROWID_", "1" )
    if (LanAdvRow ["vlanMac.vlanMacOffsetType"] == "0") then
        mac = "Same"
    else
        mac = "Unique"
    end

    printLabel ("LAN Advanced Setup")
    resTab.insertField (resultTab, "VLAN MAC Settings","")
    if(LanAdvRow ~= nil) then
        resTab.insertField (resultTab, "MAC Address for VLANs", mac or "")
    end

    resTab.insertField (resultTab, "Advanced Settings", "")    
    dhcpRow = db.getRowWhere ("dhcp","logicalIfName = 'LAN'")
    if (dhcpRow ["dhcp.enableArpBcast"] == "0") then
        arp = "Disabled"
    else
        arp = "Enabled"
    end
    if (dhcpRow ~= nil) then
        resTab.insertField (resultTab, "ARP Broadcast", arp or "")
    end		               
    resTab.print (resultTab, 0)
end


function deflanCfgInit(args)
    local configRow = {}
    local i = 0
    for k=1,4 do
        i = i + 1
        configRow["VlanPortMgmt.DefPort"..i] = db.getAttribute("VlanPortMgmt","PortName","Port"..i,"DefaultPVID") or ''
        if (configRow["VlanPortMgmt.DefPort"..i] ~= "4094") then
            configRow["VlanPortMgmt.vlanName"..i] = db.getAttribute("vlan","vlanId",configRow["VlanPortMgmt.DefPort"..i],"vlanName")
        else
            configRow["VlanPortMgmt.vlanName"..i] = nil
        end
    end
    return rowId,configRow
end

function deflanCfgInputVal(configRow)
    if (db.getAttribute("ConfigPort","_ROWID_","1","LogicalIfName") == "DMZ") then
        if (configRow["VlanPortMgmt.vlanName4"] ~= nil) then
            printCLIError ("DMZ is enabled so cannot set default vlan for port 4")
            return false
        end
    end
    local i = 0
    local vlanRows = db.getRowsWhere("vlan","portName='dummy'")
    for k=1,4 do
        i = i + 1
        local j = 0
        local flag = 0
        for m,n in pairs(vlanRows) do
            j = j + 1
            vlan = vlanRows[j]
            if(vlan ~= nil and configRow["VlanPortMgmt.vlanName"..i] ~= nil) then
                flag = 0
                if (configRow["VlanPortMgmt.vlanName"..i] == vlan["vlan.vlanName"]) then
                    break
                end
                flag = 1
            end
        end
        if (flag == 1) then
            printCLIError ("Invalid default Vlan Name,No Vlan found with Vlan Name - "..configRow["VlanPortMgmt.vlanName"..i].." for Port"..i)
            return false
        end
    end
    return true
end

function deflanCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local i = 0
    for k=1,4 do
        i = i + 1
        if (configRow["VlanPortMgmt.vlanName"..i] ~= nil) then
            configRow["VlanPortMgmt.DefPort"..i] = db.getAttribute("vlan","vlanName",configRow["VlanPortMgmt.vlanName"..i],"vlanId")
            configRow["VlanPortMgmt.vlanName"..i] = nil
        else
            configRow["VlanPortMgmt.DefPort"..i] = nil
        end
    end
    errorFlag, statusCode = vlanConfig.vlanDefVlan(configRow)
    if (errorFlag == "OK") then
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
end


function multiHomingCfgInit(args)
    local configRow = {}
    if (args[1] ~= nil) then
        configRow = db.getRowWhere("ifStaticIPAlias","LogicalIfName='LAN' AND _ROWID_=" .. args[1])
        if (configRow == nil) then
            printCLIError("This row doesn't exists")
            return -1,{}
        end
    else
        configRow["ifStaticIPAlias.LogicalIfName"] = "LAN"
        configRow["ifStaticIPAlias.PhyIfName"] = "bdg1"
        configRow["ifStaticIPAlias._ROWID_"] = "-1"
    end
    return configRow["ifStaticIPAlias._ROWID_"],configRow
end

function multiHomingCfgInputVal(configRow)
    local table = db.getTable("ifStaticIPAlias")
    if (#table >= 24) then
        printCLIError("Can't add configuration to this table. Maximum limit reached.")
        return false
    end
    if (configRow["ifStaticIPAlias.IPAddress"] == nil or
        configRow["ifStaticIPAlias.IPAddress"] == "" or
        configRow["ifStaticIPAlias.IPAddress"] == "0.0.0.0") then
        printCLIError("Invalid IP Address")
        return false
    end
    if (configRow["ifStaticIPAlias.SubnetMask"] == nil or
        configRow["ifStaticIPAlias.SubnetMask"] == "" or
        configRow["ifStaticIPAlias.SubnetMask"] == "0.0.0.0") then
        printCLIError("Invalid subnet mask")
        return false
    end
    return true
end

function multiHomingCfgSave(configRow)
    DBTable = "ifStaticIPAlias"
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local RowId = configRow["ifStaticIPAlias._ROWID_"]
    if (RowId == "-1") then
        errorFlag, statusCode = lanGroups.addLanMultiHoming(configRow)
    else
        errorFlag, statusCode = lanGroups.editLanMultiHoming(configRow, RowId, "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
end

function multiHomingDel(args)
    DBTable = "ifStaticIPAlias"
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local configRow = db.getRowWhere("ifStaticIPAlias","LogicalIfName='LAN' AND _ROWID_=" .. args[1])
    if(configRow == nil) then
        printCLIError("This row doesn't exists")
        return false
    else
        errorFlag, statusCode = lanGroups.deleteLanMultiHoming ({configRow["ifStaticIPAlias._ROWID_"]})
        -- save db if no error
	    if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    end
end


function lanMultiHomingSetupGet(args)
    local resultTab = {} -- table to store LAN setup information

    local lan2Table = db.getRows("ifStaticIPAlias","LogicalIfName","LAN")
    printLabel ("IPv4 LAN Multi-homing")
    printLabel ("Available Secondary LAN IPs :-")
    
    local i = 0
    if (#lan2Table ~= 0) then
        for k,v in pairs(lan2Table) do
            i = i + 1
            local row = lan2Table[i]
            if (row ~= nil) then
                resTab.insertField (resultTab, "Row Id", row["ifStaticIPAlias._ROWID_"])
                resTab.insertField (resultTab, "IP Address", row["ifStaticIPAlias.IPAddress"])
                resTab.insertField (resultTab, "Subnet Mask", row["ifStaticIPAlias.SubnetMask"])
            end
        end
    end
    resTab.print (resultTab, 0)
end


function ipv6multiHomingCfgInit(args)
    local configRow = {}
    if (args[1] ~= nil) then
        configRow = db.getRow("ifStaticAliasAddrTbl","_ROWID_",args[1])
        if (configRow == nil) then
            printCLIError("This row doesn't exists")
            return -1,{}
        end
    else
        configRow["ifStaticAliasAddrTbl.LogicalIfName"] = "LAN"
        configRow["ifStaticAliasAddrTbl.AddressFamily"] = "10"
        configRow["ifStaticAliasAddrTbl._ROWID_"] = "-1"
    end
    return configRow["ifStaticAliasAddrTbl._ROWID_"],configRow
end

function ipv6multiHomingCfgInputVal(configRow)
    local table = db.getTable("ifStaticAliasAddrTbl")
    if (#table >= 24) then
        printCLIError("Can't add configuration to this table. Maximum limit reached.")
        return false
    end
    if (configRow["ifStaticAliasAddrTbl.StaticIp"] == nil or
        configRow["ifStaticAliasAddrTbl.StaticIp"] == "" ) then
        printCLIError("Invalid IPv6 Address")
        return false
    end
    if (configRow["ifStaticAliasAddrTbl.PrefixLength"] == nil or
        configRow["ifStaticAliasAddrTbl.PrefixLength"] == "" ) then
        printCLIError("Invalid Prefix length")
        return false
    end
    return true
end

function ipv6multiHomingCfgSave(configRow)
    DBTable = "ifStaticAliasAddrTbl"
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local RowId = configRow["ifStaticAliasAddrTbl._ROWID_"]
    if (RowId == "-1") then
        errorFlag, statusCode = multiLan.ipv6Config(configRow, "-1", "add")
    else
        errorFlag, statusCode = multiLan.ipv6Config(configRow, RowId, "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
end

function ipv6multiHomingDel(args)
    DBTable = "ifStaticAliasAddrTbl"
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    local configRow = db.getRow("ifStaticAliasAddrTbl","_ROWID_",args[1])
    if(configRow == nil) then
        printCLIError("This row doesn't exists")
        return false
    else
        errorFlag, statusCode = multiLan.ipv6Config (configRow, configRow["ifStaticAliasAddrTbl._ROWID_"], "delete")
        -- save db if no error
	    if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    end
end


function lanMultiHoming6SetupGet(args)
    local resultTab = {} -- table to store LAN setup information

    local lan2Table = db.getTable("ifStaticAliasAddrTbl")
    printLabel ("IPv6 LAN Multi-homing")
    printLabel ("Available Secondary LAN IPs :-")
    
    local i = 0
    if (#lan2Table ~= 0) then
        for k,v in pairs(lan2Table) do
            i = i + 1
            local row = lan2Table[i]
            if (row ~= nil) then
                resTab.insertField (resultTab, "Row Id", row["ifStaticAliasAddrTbl._ROWID_"])
                resTab.insertField (resultTab, "IPv6 Address", row["ifStaticAliasAddrTbl.StaticIp"])
                resTab.insertField (resultTab, "Prefix Length", row["ifStaticAliasAddrTbl.PrefixLength"])
            end
        end
    end
    resTab.print (resultTab, 0)
end

function prefixDelegationInit (args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    local configRow = {}
    if(args[1] ~= nil) then
        configRow = db.getRow("dhcpv6sLANPrefixPool","_ROWID_", args[1])
        if (configRow == nil) then
            printCLIError ("Row doesn't exists")
            return -1, {}
        end
    else
        configRow ["dhcpv6sLANPrefixPool._ROWID_"] = "-1"
    end
    return configRow ["dhcpv6sLANPrefixPool._ROWID_"], configRow
end

--
-- This routine saves the modified dhcpv6sLANPrefixPool information.
function prefixDelegationSave (configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANPrefixPool"

    if (configRow ["dhcpv6sLANPrefixPool._ROWID_"] == "-1") then
        errorFlag, statusCode = lan.prefixDelegationConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = lan.prefixDelegationConfig(configRow, configRow ["dhcpv6sLANPrefixPool._ROWID_"], "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

function prefixDelegationInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 LAN Prefix Delegation.")
        return false
    end
    if (configRow ["dhcpv6sLANPrefixPool.delegationPrefix"] == "" or configRow ["dhcpv6sLANPrefixPool.delegationPrefix"] == nil) then
         printCLIError("Please Enter valid Delegation Prefix.")
         return false
    end
    if (configRow ["dhcpv6sLANPrefixPool.delegationPrefixLen"] == "" or configRow ["dhcpv6sLANPrefixPool.delegationPrefixLen"] == nil)then
        printCLIError ("Please Enter valid Prefix Length.")
        return false
    end

    return true
end
--
-- This routine delete the modified dhcpv6sLANPrefixPool configuration information.
function prefixDelegationDel (args)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANPrefixPool"

    local configRow = db.getRow("dhcpv6sLANPrefixPool","_ROWID_", args[1])
    if (configRow == nil) then
        printCLIError ("Row doesn't exists")
        return false
    end
    row = {}
    row["_ROWID_"] = configRow ["dhcpv6sLANPrefixPool._ROWID_"]
    errorFlag, statusCode = lan.lanPrefixPoolsDelete(configRow)

    -- save db if no error
    if (errorFlag == "OK") then 
        db.save() 
        print ("Row deleted")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end
