-- sslvpnCLI.lua
-- Bharadwaj Avva
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 27JJun, sig fixed SPR 32982
-- 10may12,nik fixed SPR 31512,31513
-- 26mar12,akr added command for configuring sslvpn client for ipv6
-- 22mar12,sig fixed spr 30693
-- 20mar12,nik fixed SPR 30667
-- 19mar12,akr fixed 30654 
-- 15mar12,nik changes to configure Ipv6 client route
-- 13mar12,akr fixed 30497 
-- 07mar12,sig fixed SPR 30373
-- 05jan12,nik fixed SPR #28861
-- 30nov11,akr fixed 28150
-- 8nov11,akr fixed 27645
-- 01nov11,akr Fixed 27423 
-- 26Feb11,rac Fixed 21877
-- 25Feb11,rac Fixed 21874
-- 02Feb11,bhj Fixed 21410
-- 02Feb11,bhj Fixed 21400
-- 21Jan11,bhj modified sslvpn validations
-- 19Jan11,rac modified to add users module
-- 21Dec10,bhj written
--
-- Description
-- CLISH sslvpn LUA routines
--

require "teamf1lualib/sslvpn"
require "teamf1lualib/validations"
require "teamf1lualib/sslAdmin"

function DeleteStaleInput()
	__CLISH_CUR_CONFIG_ROW = {} -- current config info is now discarded
 	__CLISH_EDIT_CONFIG_ROW = {} -- current edit config info is discarded
	return true
end

function sslvpnportallayoutCfgInit(args)
	local configRow = {}
	if (args[1] ~= nil) then
		configRow = db.getRow("SSLVPNPortalLayout", "SSLVPNPortalLayout._ROWID_", args[1])
		if (configRow == nil)then
			printCLIError("Row does not exist")
			return -1,{}
		else
			return 1,configRow
		end
	else
		configRow["SSLVPNPortalLayout.DisplayBanner"] = "0"
		configRow["SSLVPNPortalLayout.EnableHTTPMetaTags"] = "0"
		configRow["SSLVPNPortalLayout.EnableActiveXWebCacheCleaner"] = "0"
		configRow["SSLVPNPortalLayout.EnableVPNTunnel"] = "1"
		configRow["SSLVPNPortalLayout.EnablePortForwarding"] = "0"
		return 1,configRow
	end
end

function sslvpnportallayoutCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPortalLayout"
	RowId=configRow["SSLVPNPortalLayout._ROWID_"]
	if (RowId == nil or RowId == "") then
		errorFlag, statusCode = sslvpn.portalLayoutConfig(configRow, "-1", "add")
	--if edit
	else
		if (db.getRow("SSLVPNPortalLayout", "SSLVPNPortalLayout._ROWID_", RowId) == nil)then
			printCLIError("Row does not exist")
 			return errorFlag, "Row does not exist"
		end
        local name = db.getAttribute ("SSLVPNPortalLayout","SSLVPNPortalLayout._ROWID_", RowId, "PortalLayoutName")
        if (configRow["SSLVPNPortalLayout.PortalLayoutName"] ~= name) then
            printCLIError("Portal Layout name can't be modified")
            configRow["SSLVPNPortalLayout.PortalLayoutName"] = name
        end
		errorFlag, statusCode = sslvpn.portalLayoutConfig(configRow, RowId, "edit")
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnportallayoutCfgInputVal(configRow)
	if (configRow["SSLVPNPortalLayout.PortalLayoutName"] == nil or configRow["SSLVPNPortalLayout.PortalLayoutName"] == "")then
		printCLIError("Invalid portal_name value")
		return false
	end
	--if ( configRow["SSLVPNPortalLayout.DefaultPortal"] == '1') then
		--printCLIError("Default Portal-Layout can't be changed !!")
		--return false
	--end
	return db.typeAndRangeValidate(configRow)
end

function sslvpnportallayoutCfgDel(args)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	rows = {args[1]}

	if (db.getRow("SSLVPNPortalLayout", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.deleteLayouts (rows)

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnportallayoutCfgDefault(args)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	configRow = db.getRow("SSLVPNPortalLayout", "SSLVPNPortalLayout._ROWID_", args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.setDefault (args[1])

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnportforwardingappsCfgInit(args)
	local configRow = {}
	return 1,configRow
end

function sslvpnportforwardingappsCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPortForwardingAppsMap"
	errorFlag, statusCode = sslvpn.portForwardConfig (configRow, -1, "add")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnportforwardingappsCfgInputVal(configRow)
	if (configRow["SSLVPNPortForwardingAppsMap.ServerIP"] == nil or configRow["SSLVPNPortForwardingAppsMap.ServerIP"] == "")then
		printCLIError("Invalid serverip value")
		return false
	end
	if (configRow["SSLVPNPortForwardingAppsMap.Port"] == nil or configRow["SSLVPNPortForwardingAppsMap.Port"] == "")then
		printCLIError("Invalid port value")
		return false
	end

	return (validations.ipAddressValidate(configRow["SSLVPNPortForwardingAppsMap.ServerIP"],"","IP Address error") and validations.rangeValidate(0,65535,configRow["SSLVPNPortForwardingAppsMap.Port"],"","TCP Port error"))
end

function sslvpnportforwardingappsCfgDel(args)
	rows = {args[1]}
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPortForwardingAppsMap"

	if (db.getRow("SSLVPNPortForwardingAppsMap", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.portForwardConfig (rows, "-1", "delete")
	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnportforwardinghostCfgInit(args)
	local configRow = {}
	return 1,configRow
end

function sslvpnportforwardinghostCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPortForwardingFQDNMap"
	errorFlag, statusCode = sslvpn.portForwardConfig (configRow, -1, "add")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnportforwardinghostCfgInputVal(configRow)
	if (configRow["SSLVPNPortForwardingFQDNMap.ServerIP"] == nil or configRow["SSLVPNPortForwardingFQDNMap.ServerIP"] == "")then
		printCLIError("Invalid serverip value")
		return false
	end
	if (configRow["SSLVPNPortForwardingFQDNMap.FQDNName"] == nil or configRow["SSLVPNPortForwardingFQDNMap.FQDNName"] == "")then
		printCLIError("Invalid domain_name value")
		return false
	end

	return validations.ipAddressValidate(configRow["SSLVPNPortForwardingFQDNMap.ServerIP"],"","IP Address error")
end

function sslvpnportforwardinghostCfgDel(args)
	rows = {args[1]}
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPortForwardingFQDNMap"

	if (db.getRow("SSLVPNPortForwardingFQDNMap", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.portForwardConfig (rows, "-1", "delete")
	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnresourceCfgInit(args)
	local configRow = {}
	return 1,configRow
end

function sslvpnresourceCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNResource"
	errorFlag, statusCode = sslvpn.resourceConfig(configRow, -1, "add")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnresourceCfgInputVal(configRow)
	if (configRow["SSLVPNResource.ResourceName"] == nil or configRow["SSLVPNResource.ResourceName"] == "")then
		printCLIError("Invalid resource_name value")
		return false
	end
	if (configRow["SSLVPNResource.ServiceType"] == nil or configRow["SSLVPNResource.ServiceType"] == "")then
		printCLIError("Invalid service_type value")
		return false
	end

	return true
end

function sslvpnresourceCfgDel(args)
	rows = {args[1]}
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""

    if (db.getRow("SSLVPNResource", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
    else
        local count = db.getAttribute("SSLVPNResource", "_ROWID_", args[1], "UseCount")
        if(tonumber(count) > 0) then
            printCLIError("Resource in use, Cannot delete the Resource")
            return false
        end
    end
	
	errorFlag, statusCode = sslvpn.deleteResources (rows)

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnresourceobjectCfgInit(args)
	local configRow = {}
	configRow["SSLVPNResourceObject.ResourceName"] = args[1]
    configRow["SSLVPNResourceObject.IcmpBlock"] = "no"
	return 1,configRow
end

function sslvpnresourceobjectCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNResourceObject"
    if (configRow["SSLVPNResourceObject.ObjectType"] == "1" and configRow["SSLVPNResourceObject.AddressType"] == "2")then
        configRow["SSLVPNResourceObject.MaskLength"] = "128"
    end
	errorFlag, statusCode = sslvpn.resourceConfig(configRow, -1, "add")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnresourceobjectCfgInputVal(configRow)
	if (configRow["SSLVPNResourceObject.ObjectType"] == nil or configRow["SSLVPNResourceObject.ObjectType"] == "")then
		printCLIError("Invalid object_type value")
		return false
	end
    if (configRow["SSLVPNResourceObject.AddressType"] == nil or configRow["SSLVPNResourceObject.AddressType"] == "")then
		printCLIError("Invalid ip_version value")
		return false
	end
    if (configRow["SSLVPNResourceObject.AddressType"] == "2" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure ip_version option as IPv6.")
        return false
    end
    if (configRow["SSLVPNResourceObject.AddressType"] == "1") then
	    if (configRow["SSLVPNResourceObject.ObjectAddress"] == nil or configRow["SSLVPNResourceObject.ObjectAddress"] == "")then
		    printCLIError("Invalid object_address value")
		    return false
        end
    elseif (configRow["SSLVPNResourceObject.AddressType"] == "2") then
        if (configRow["SSLVPNResourceObject.ObjectAddress6"] == nil or configRow["SSLVPNResourceObject.ObjectAddress6"] == "")then
		    printCLIError("Invalid object_address6 value")
		    return false
        end
	end
	if (configRow["SSLVPNResourceObject.StartPort"] == nil or configRow["SSLVPNResourceObject.StartPort"] == "")then
		printCLIError("Invalid start_port value")
		return false
	end
	if (configRow["SSLVPNResourceObject.EndPort"] == nil or configRow["SSLVPNResourceObject.EndPort"] == "")then
		printCLIError("Invalid end_port value")
		return false
	end
    if (tonumber(configRow["SSLVPNResourceObject.StartPort"]) > tonumber(configRow["SSLVPNResourceObject.EndPort"])) then
        printCLIError("Start Port Number should not greater than End Port Number")
        return false
    end
	if (configRow["SSLVPNResourceObject.ObjectType"] == "2")then
        if (configRow["SSLVPNResourceObject.AddressType"] == "1") then
		    if (configRow["SSLVPNResourceObject.MaskLength"] == nil or configRow["SSLVPNResourceObject.MaskLength"] == "" or
                tonumber(configRow["SSLVPNResourceObject.MaskLength"]) < 1 or tonumber(configRow["SSLVPNResourceObject.MaskLength"]) > 32)then
			    printCLIError("Invalid mask_length value,Enter a value between 1 - 32")
			    return false
            end
        elseif (configRow["SSLVPNResourceObject.AddressType"] == "2") then
            if (configRow["SSLVPNResourceObject.MaskLength"] == nil or configRow["SSLVPNResourceObject.MaskLength"] == "" or
                tonumber(configRow["SSLVPNResourceObject.MaskLength"]) < 1 or tonumber(configRow["SSLVPNResourceObject.MaskLength"]) > 128)then
			    printCLIError("Invalid mask_length value,Enter a value between 1 - 128")
			    return false
            end
        end
	end

	return db.typeAndRangeValidate(configRow)
end

function sslvpnresourceobjectCfgDel(args)
	rows = {args[1]}
	rName = db.getAttribute("SSLVPNResourceObject", "_ROWID_", args[1], "ResourceName")
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNResourceObject"

	if (db.getRow("SSLVPNResourceObject", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.deleteDefinedResource (rows, rName)

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
	return errorFlag, statusMessage
end

function sslvpnpolicyCfgInit(args)
	local configRow = {}
	if (args[1] ~= nil) then
		configRow = db.getRow("SSLVPNPolicy", "SSLVPNPolicy._ROWID_", args[1])
		if (configRow == nil)then
			printCLIError("Row does not exist")
			return -1,{}
		else
            if (configRow["SSLVPNPolicy.IPVersion"] == "2") then
                configRow["SSLVPNPolicy.Address6"] = configRow["SSLVPNPolicy.Address"]
                configRow["SSLVPNPolicy.PrefixLength"] = configRow["SSLVPNPolicy.MaskLength"]
            end
			return 1,configRow
		end
	else
        configRow["SSLVPNPolicy.IcmpBlock"] = "no"
		return 1,configRow
	end
end

function sslvpnpolicyCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPolicy"
    if (configRow["SSLVPNPolicy.IPVersion"] == "2") then
        configRow["SSLVPNPolicy.Address"] = configRow["SSLVPNPolicy.Address6"]
        configRow["SSLVPNPolicy.MaskLength"] = configRow["SSLVPNPolicy.PrefixLength"]
    end
    configRow["SSLVPNPolicy.Address6"] = nil
    configRow["SSLVPNPolicy.PrefixLength"] = nil

	RowId=configRow["SSLVPNPolicy._ROWID_"]
	if (RowId == nil or RowId == "") then
		errorFlag, statusCode = sslvpn.policyConfig(configRow, RowId, "add")
	--if edit
	else
		if (db.getRow("SSLVPNPolicy", "SSLVPNPolicy._ROWID_", RowId) == nil)then
			printCLIError("Row does not exist")
 			return errorFlag, "Row does not exist"
		end
        local policyRow = db.getRow("SSLVPNPolicy","SSLVPNPolicy._ROWID_",RowId)
        if(policyRow["SSLVPNPolicy.PolicyName"] ~= configRow["SSLVPNPolicy.PolicyName"]) then
            configRow["SSLVPNPolicy.PolicyName"] = policyRow["SSLVPNPolicy.PolicyName"]
            printCLIError("PolicyName cannot be modified")
        end
        if(policyRow["SSLVPNPolicy.ServiceType"] ~= configRow["SSLVPNPolicy.ServiceType"]) then
            configRow["SSLVPNPolicy.ServiceType"] = policyRow["SSLVPNPolicy.ServiceType"]
            printCLIError("ServiceType cannot be modified")
        end
        if(policyRow["SSLVPNPolicy.DestinationObjectType"] ~= configRow["SSLVPNPolicy.DestinationObjectType"]) then
            configRow["SSLVPNPolicy.DestinationObjectType"] = policyRow["SSLVPNPolicy.DestinationObjectType"]
            printCLIError("DestinationObjectType cannot be modified")
        end
		errorFlag, statusCode = sslvpn.policyConfig(configRow, RowId, "edit")
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnpolicyCfgInputVal(configRow)
	if (configRow["SSLVPNPolicy.PolicyType"] == nil or configRow["SSLVPNPolicy.PolicyType"] == "")then
		printCLIError("Invalid policy_type value")
		return false
	end
    if (configRow["SSLVPNPolicy.PolicyType"] ~= "global") then
	    if (configRow["SSLVPNPolicy.PolicyOwnerName"] == nil or configRow["SSLVPNPolicy.PolicyOwnerName"] == "")then
		    printCLIError("Invalid policy_owner value")
		    return false
	    end
    else
        configRow["SSLVPNPolicy.PolicyOwnerName"] = "global"
    end
    if ((configRow["SSLVPNPolicy.IPVersion"] == nil or configRow["SSLVPNPolicy.IPVersion"] == "") and 
        (configRow["SSLVPNPolicy.DestinationObjectType"] == "2" or configRow["SSLVPNPolicy.DestinationObjectType"] == "3"))then
		printCLIError("Invalid ip_version value")
		return false
	end
    if (configRow["SSLVPNPolicy.IPVersion"] == "2" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure ip_version option as IPv6.")
        return false
    end
	local policyOwnerName = configRow["SSLVPNPolicy.PolicyOwnerName"]
	if (configRow["SSLVPNPolicy.PolicyType"] == "group")then
		if (db.getRow("USERDBGroups", "GroupName", policyOwnerName) == nil)then
			printCLIError("Invalid policy_owner value. No such group found")
			return false
		end
	elseif(configRow["SSLVPNPolicy.PolicyType"] == "user")then
		if (db.getRow("USERDBUsers", "UserName", policyOwnerName) == nil)then
			printCLIError("Invalid policy_owner value. No such user found")
			return false
		end
	end
	if (configRow["SSLVPNPolicy.DestinationObjectType"] == nil or configRow["SSLVPNPolicy.DestinationObjectType"] == "")then
		printCLIError("Invalid destination_objecttype value")
		return false
	end
	if (configRow["SSLVPNPolicy.PolicyName"] == nil or configRow["SSLVPNPolicy.PolicyName"] == "")then
		printCLIError("Invalid policy_name value")
		return false
	end
	if (configRow["SSLVPNPolicy.DestinationObjectType"] == "1")then
		if (configRow["SSLVPNPolicy.ResourceName"] == nil or configRow["SSLVPNPolicy.ResourceName"] == "")then
			printCLIError("Invalid resource_name value")
			return false
		end
	elseif (configRow["SSLVPNPolicy.DestinationObjectType"] == "2")then
        if (configRow["SSLVPNPolicy.IPVersion"] == "1") then
		    if (configRow["SSLVPNPolicy.Address"] == nil or configRow["SSLVPNPolicy.Address"] == "")then
			    printCLIError("Invalid policy_address value")
			    return false
            end
        elseif (configRow["SSLVPNPolicy.IPVersion"] == "2") then
            if (configRow["SSLVPNPolicy.Address6"] == nil or configRow["SSLVPNPolicy.Address6"] == "")then
			    printCLIError("Invalid policy_address6 value")
			    return false
            end
		end
		if (configRow["SSLVPNPolicy.StartPort"] == nil or configRow["SSLVPNPolicy.StartPort"] == "")then
			printCLIError("Invalid start_port value")
			return false
		end
		if (configRow["SSLVPNPolicy.EndPort"] == nil or configRow["SSLVPNPolicy.EndPort"] == "")then
			printCLIError("Invalid end_port value")
			return false
		end
        if (tonumber(configRow["SSLVPNPolicy.StartPort"]) > tonumber(configRow["SSLVPNPolicy.EndPort"])) then
            printCLIError("Start Port Number should not greater than End Port Number")
            return false
        end
		if (configRow["SSLVPNPolicy.ServiceType"] == nil or configRow["SSLVPNPolicy.ServiceType"] == "")then
			printCLIError("Invalid service_type value")
			return false
		end
	elseif (configRow["SSLVPNPolicy.DestinationObjectType"] == "3")then
        if (configRow["SSLVPNPolicy.IPVersion"] == "1") then
		    if (configRow["SSLVPNPolicy.Address"] == nil or configRow["SSLVPNPolicy.Address"] == "")then
			    printCLIError("Invalid policy_address value")
			    return false
		    end
		    if (configRow["SSLVPNPolicy.MaskLength"] == nil or configRow["SSLVPNPolicy.MaskLength"] == "" or
                tonumber(configRow["SSLVPNPolicy.MaskLength"]) < 1 or tonumber(configRow["SSLVPNPolicy.MaskLength"]) > 32)then
			    printCLIError("Invalid policy_masklength value,Enter a value between 1 - 32")
			    return false
		    end
        elseif (configRow["SSLVPNPolicy.IPVersion"] == "2") then
            if (configRow["SSLVPNPolicy.Address6"] == nil or configRow["SSLVPNPolicy.Address6"] == "")then
			    printCLIError("Invalid policy_address6 value")
			    return false
		    end
		    if (configRow["SSLVPNPolicy.PrefixLength"] == nil or configRow["SSLVPNPolicy.PrefixLength"] == "")then
			    printCLIError("Invalid policy_ipv6_prefix_length value,Enter a value between 1 - 128")
			    return false
		    end
        end
		if (configRow["SSLVPNPolicy.StartPort"] == nil or configRow["SSLVPNPolicy.StartPort"] == "")then
			printCLIError("Invalid start_port value")
			return false
		end
		if (configRow["SSLVPNPolicy.EndPort"] == nil or configRow["SSLVPNPolicy.EndPort"] == "")then
			printCLIError("Invalid end_port value")
			return false
		end
        if (tonumber(configRow["SSLVPNPolicy.StartPort"]) > tonumber(configRow["SSLVPNPolicy.EndPort"])) then
            printCLIError("Start Port Number should not greater than End Port Number")
            return false
        end
		if (configRow["SSLVPNPolicy.ServiceType"] == nil or configRow["SSLVPNPolicy.ServiceType"] == "")then
			printCLIError("Invalid service_type value")
			return false
		end
	elseif (configRow["SSLVPNPolicy.DestinationObjectType"] == "4")then
		if (configRow["SSLVPNPolicy.StartPort"] == nil or configRow["SSLVPNPolicy.StartPort"] == "")then
			printCLIError("Invalid start_port value")
			return false
		end
		if (configRow["SSLVPNPolicy.EndPort"] == nil or configRow["SSLVPNPolicy.EndPort"] == "")then
			printCLIError("Invalid end_port value")
			return false
		end
        if (tonumber(configRow["SSLVPNPolicy.StartPort"]) > tonumber(configRow["SSLVPNPolicy.EndPort"])) then
            printCLIError("Start Port Number should not greater than End Port Number")
            return false
        end
		if (configRow["SSLVPNPolicy.ServiceType"] == nil or configRow["SSLVPNPolicy.ServiceType"] == "")then
			printCLIError("Invalid service_type value")
			return false
		end
	end
	if (configRow["SSLVPNPolicy.Permission"] == nil or configRow["SSLVPNPolicy.Permission"] == "")then
		printCLIError("Invalid policy_permission value")
		return false
	end

	return (validations.isEmpty(configRow["SSLVPNPolicy.PolicyName"],"Policy Name error") and
			validations.checkboxValidate(configRow["SSLVPNPolicy.DestinationObjectType"],"2",2) and
			--validations.ipAddressValidate(configRow["SSLVPNPolicy.Address"],"","IP Address error") and
			validations.rangeValidate(0,65535,configRow["SSLVPNPolicy.StartPort"],configRow["SSLVPNPolicy.EndPort"],"Port start/end error") and
			validations.checkboxValidate(configRow["SSLVPNPolicy.DestinationObjectType"],"3",3) and
			--validations.ipAddressValidate(configRow["SSLVPNPolicy.Address"],"","IP address error") and
			--validations.rangeValidate(0,32,configRow["SSLVPNPolicy.MaskLength"],"","Mask Length error") and
			validations.rangeValidate(0,65535,configRow["SSLVPNPolicy.StartPort"],configRow["SSLVPNPolicy.EndPort"],"Port start/end error") and
			validations.checkboxValidate(configRow["SSLVPNPolicy.DestinationObjectType"],"4",1) and
			validations.rangeValidate(0,65535,configRow["SSLVPNPolicy.StartPort"],configRow["SSLVPNPolicy.EndPort"],"Port start/end error"))
end

function sslvpnpolicyCfgDel(args)
	rows = {args[1]}
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNPolicy"

	if (db.getRow("SSLVPNPolicy", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.deletePolicy(rows)

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnclientCfgInit(args)
	local configRow = db.getRow("SSLVPNClientConf","_ROWID_", "1")
    if (configRow == nil) then
        configRow = {}
        configRow["SSLVPNClientConf.lcpTimeout"] = "60"
        configRow["SSLVPNClientConf.ForceIPv6"] = "0"
    end
    return 1,configRow
end

function sslvpnclientCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNClientConf"
	inputTable = db.getRow("SSLVPNClientConf","_ROWID_", "1")
	if(inputTable == nil) then
	   	errorFlag, statusCode = sslvpn.sslVpnClientConfig (configRow, "-1", "add")
	else
	   	errorFlag, statusCode = sslvpn.sslVpnClientConfig (configRow, "1", "edit")
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnclientCfgInputVal(configRow)
	if (configRow["SSLVPNClientConf.EnableFullTunnel"] == nil or configRow["SSLVPNClientConf.EnableFullTunnel"] == "") then
		printCLIError("Invalid enable_fulltunnel value")
		return false
	end
--[[    if (configRow["SSLVPNClientConf.ForceIPv6"] == nil or configRow["SSLVPNClientConf.ForceIPv6"] == "") then
		printCLIError("Invalid force_ipv6_enable value")
		return false
	end
]]
	if (configRow["SSLVPNClientConf.BeginClientAddress"] == nil or configRow["SSLVPNClientConf.BeginClientAddress"] == "" or 
        configRow["SSLVPNClientConf.BeginClientAddress"] == "0.0.0.0") then
	    printCLIError("Invalid begin_clientaddress value")
	    return false
	end
	if (configRow["SSLVPNClientConf.EndClientAddress"] == nil or configRow["SSLVPNClientConf.EndClientAddress"] == "" or
        configRow["SSLVPNClientConf.EndClientAddress"] == "0.0.0.0") then
	    printCLIError("Invalid end_clientaddress value")
	    return false
    end
    --if (configRow["SSLVPNClientConf.EnableFullTunnel"] == "0") then
        --configRow["SSLVPNClientConf.BeginClientAddress6"] = nil
	    --configRow["SSLVPNClientConf.EndClientAddress6"] = nil
	--end

	return (validations.ipAddressValidate(configRow["SSLVPNClientConf.PrimaryDNSServer"],"","Primary DNS Error","skip") and
			validations.ipAddressValidate(configRow["SSLVPNClientConf.SecondaryDNSServer"],"","Secondary DNS error","skip") and
			validations.ipRangeValidate(configRow["SSLVPNClientConf.BeginClientAddress"],configRow["SSLVPNClientConf.EndClientAddress"],"Start/End IP error"))	
end

function sslvpnrouteCfgInit(args)
	local configRow = {}
	return 1,configRow
end

function sslvpnrouteCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNClientRoute"
    if (configRow["SSLVPNClientRoute.IPVersion"] == "2") then
        configRow["SSLVPNClientRoute.DestinationNetwork"] = configRow["SSLVPNClientRoute.DestinationNetwork6"]
        configRow["SSLVPNClientRoute.SubnetMask"] = configRow["SSLVPNClientRoute.SubnetMask6"]
    end
    configRow["SSLVPNClientRoute.DestinationNetwork6"] = nil
    configRow["SSLVPNClientRoute.SubnetMask6"] = nil
   	errorFlag, statusCode = sslvpn.sslClientRouteConfig (configRow, "-1", "add")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

function sslvpnrouteCfgInputVal(configRow)
	local fullTunnelStatus = db.getAttribute("SSLVPNClientConf", "_ROWID_", "1", "EnableFullTunnel")
	if (fullTunnelStatus == "1")then
		printCLIError("Enable Split Tunnel Support to configure Client Routes")
		return false
	end
    if (configRow["SSLVPNClientRoute.IPVersion"] == nil or configRow["SSLVPNClientRoute.IPVersion"] == "") then
        printCLIError("Invalid ip_version value")
		return false
	end
    if (configRow["SSLVPNClientRoute.IPVersion"] == "1") then
	    if (configRow["SSLVPNClientRoute.DestinationNetwork"] == nil or configRow["SSLVPNClientRoute.DestinationNetwork"] == "")then
		    printCLIError("Invalid destination_network value")
		    return false
	    end
	    if (configRow["SSLVPNClientRoute.SubnetMask"] == nil or configRow["SSLVPNClientRoute.SubnetMask"] == "")then
		    printCLIError("Invalid subnet_mask value")
		    return false
	    end
    elseif (configRow["SSLVPNClientRoute.IPVersion"] == "2") then
        if (configRow["SSLVPNClientRoute.DestinationNetwork6"] == nil or configRow["SSLVPNClientRoute.DestinationNetwork6"] == "")then
		    printCLIError("Invalid IPv6 destination_network6 value")
		    return false
	    end
	    if (configRow["SSLVPNClientRoute.SubnetMask6"] == nil or configRow["SSLVPNClientRoute.SubnetMask6"] == "")then
		    printCLIError("Invalid prefix_length value")
		    return false
	    end
    end

	return validations.ipAddressValidate(configRow["SSLVPNClientRoute.DestinationNetwork"],configRow["SSLVPNClientRoute.SubnetMask"],"IP/Subnet Mask Error")
end

function sslvpnrouteCfgDel(args)
	rows = {args[1]}
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="SSLVPNClientRoute"

	if (db.getRow("SSLVPNClientRoute", "_ROWID_", args[1]) == nil)then
		printCLIError("Row does not exist")
 		return errorFlag, "Row does not exist"
	end
	errorFlag, statusCode = sslvpn.sslClientRouteConfig (rows, "-1", "delete")

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	printCLIError(statusMessage or statusCode)
 	return errorFlag, statusMessage
end

--function for showing sslvpn logs
function sslvpnLogsShow()
	os.execute("sqlite3 /tmp/logging.db 'select textMessage from eventLog'|grep SSLVPN|tail -n50")
	return true
end
function sslvpnClientShow(args)
	configRow = db.getRow("SSLVPNClientConf","_ROWID_", "1")
	if(configRow == nil) then
		configRow = db.getDefaults(true, "SSLVPNClientConf")
	end
	local resultTab = {}
--	local i = 0

	resTab.insertField (resultTab,"Enable Full Tunnel Support", (((configRow["SSLVPNClientConf.EnableFullTunnel"] or "") == "0") and "No" or "Yes"))
--    resTab.insertField (resultTab,"Force IPv6", (((configRow["SSLVPNClientConf.ForceIPv6"] or "") == "0") and "Disable" or "Enable"))
	resTab.insertField (resultTab,"DNS Suffix", (configRow["SSLVPNClientConf.DNSSuffix"] or ""))
	resTab.insertField (resultTab,"Primary DNS Server", (configRow["SSLVPNClientConf.PrimaryDNSServer"] or ""))
	resTab.insertField (resultTab,"Secondary DNS Server", (configRow["SSLVPNClientConf.SecondaryDNSServer"] or ""))
	resTab.insertField (resultTab,"Client Address Range Begin", (configRow["SSLVPNClientConf.BeginClientAddress"] or ""))
	resTab.insertField (resultTab,"Client Address Range End", (configRow["SSLVPNClientConf.EndClientAddress"] or ""))
	--resTab.insertField (resultTab,"LCP Timeout", (configRow["SSLVPNClientConf.lcpTimeout"] or ""))
	printLabel("SSL VPN Client(IPv4)")
	resTab.print (resultTab, 0)
	--ipv6 configuration
	local resultTab = {}
	resTab.insertField (resultTab,"Enable Full Tunnel Support", (((configRow["SSLVPNClientConf.EnableFullTunnel"] or "") == "0") and "No" or "Yes"))
--    resTab.insertField (resultTab,"Force IPv6", (((configRow["SSLVPNClientConf.ForceIPv6"] or "") == "0") and "Disable" or "Enable"))
	resTab.insertField (resultTab,"DNS Suffix", (configRow["SSLVPNClientConf.DNSSuffix"] or ""))
	resTab.insertField (resultTab,"Primary DNS Server", (configRow["SSLVPNClientConf.PrimaryDNSServer"] or ""))
	resTab.insertField (resultTab,"Secondary DNS Server", (configRow["SSLVPNClientConf.SecondaryDNSServer"] or ""))
	resTab.insertField (resultTab,"Client Address Range Begin", (configRow["SSLVPNClientConf.BeginClientAddress6"] or ""))
	resTab.insertField (resultTab,"Client Address Range End", (configRow["SSLVPNClientConf.EndClientAddress6"] or ""))

	printLabel("SSL VPN Client(IPv6)")
	resTab.print (resultTab, 0)
end

function sslvpnRouteShow(args)
	local table = db.getTable("SSLVPNClientRoute")
	local resultTab = {}
	local i = 0

	for k,v in pairs(table) do
		i = i + 1							
		resTab.insertField (resultTab,"Row Id",v["SSLVPNClientRoute._ROWID_"])
		resTab.insertField (resultTab,"Destination Network", (v["SSLVPNClientRoute.DestinationNetwork"] or ""))
		resTab.insertField (resultTab,"Subnet Mask", (v["SSLVPNClientRoute.SubnetMask"] or ""))
	end
	printLabel("Configured Client Routes")
	resTab.print (resultTab, 0)
end

function sslvpnPolicyShow(args)
	local table = db.getTable("SSLVPNPolicy")
	local resultTab = {}
	local i = 0

	for k,v in pairs(table) do
		i = i + 1							
		resTab.insertField (resultTab,"Row Id",v["SSLVPNPolicy._ROWID_"])
		resTab.insertField (resultTab,"Policy Name", (v["SSLVPNPolicy.PolicyName"] or ""))
        resTab.insertField (resultTab,"Policy Type", (v["SSLVPNPolicy.PolicyType"] or ""))
        local ServiceName = ""
		if (v["SSLVPNPolicy.DestinationObjectType"] == "1") then
		    local resSevType = db.getAttribute ("SSLVPNResource", "ResourceName", v["SSLVPNPolicy.ResourceName"], "ServiceType") or ''
			if(resSevType == "VIRTUAL-PASSAGE") then
			    ServiceName = "VPN Tunnel"
			elseif(resSevType == "VIRTUAL-TRANSPORT") then
			    ServiceName = "Port Forwarding"
			elseif(resSevType == "all") then
			    ServiceName = "All"
			end
		elseif(v["SSLVPNPolicy.ServiceType"] == "VIRTUAL-PASSAGE") then
		    ServiceName = "VPN Tunnel"
		elseif(v["SSLVPNPolicy.ServiceType"] == "VIRTUAL-TRANSPORT") then
		    ServiceName = "Port Forwarding"
		elseif(v["SSLVPNPolicy.ServiceType"] == "all") then
		    ServiceName = "All"
		end
		resTab.insertField (resultTab,"Service Type",ServiceName)
		resTab.insertField (resultTab,"Destination Object", (v["SSLVPNPolicy.DestinationObject"] or ""))
		resTab.insertField (resultTab,"Permission", (v["SSLVPNPolicy.Permission"] or ""))
	end
	printLabel("SSL VPN Policies")
	resTab.print (resultTab, 0)
end

function sslvpnPortalShow(args)
	local table = db.getTable("SSLVPNPortalLayout")
	local resultTab = {}
	local i = 0						
	for k,v in pairs(table) do
		i = i + 1							
		local row = table[i]
        local portalName = ""
		resTab.insertField (resultTab,"Row Id",row["SSLVPNPortalLayout._ROWID_"])
        portalName =  row["SSLVPNPortalLayout.PortalLayoutName"]
        if ( row["SSLVPNPortalLayout.DefaultPortal"] == "1") then
			if( db.existsRow ("SSLVPNPortalLayout", "DefaultPortal", "2")) then
                portalName =  row["SSLVPNPortalLayout.PortalLayoutName"]
			else
                portalName =  row["SSLVPNPortalLayout.PortalLayoutName"].."*"
			end
		end
		if ( row["SSLVPNPortalLayout.DefaultPortal"] == "2") then
            portalName =  row["SSLVPNPortalLayout.PortalLayoutName"].."*"
		end
        local bannerMessage = ""
	    if(row["SSLVPNPortalLayout.BannerMessage"] ~= nil) then
		    bannerMessage = row["SSLVPNPortalLayout.BannerMessage"]
            if (#bannerMessage > 30) then
                bannerMessage = string.sub(bannerMessage,1,27) .. "..."
            end
		end
    	local wanip = "0.0.0.0"
		local wanModeConfig = db.getRow("WanMode", "_ROWID_", "1")		
		local LogIfName = "WAN1"
		if (wanModeConfig ~= nil) then
			if (wanModeConfig["WanMode.Wanmode"] == "0" or wanModeConfig["WanMode.Wanmode"] == "2") then
				LogIfName = wanModeConfig["WanMode.DedicatedLogicalIfName"]
			end
		end
		if (LogIfName == nil) then
			LogIfName = "WAN1"
		end
		local wanconf = db.getRowWhere ("ipAddressTable","LogicalIfName='" .. LogIfName .. "' and addressFamily=2")
    	if (wanconf ~= nil) then
    		wanip = wanconf["ipAddressTable.ipAddress"] or wanip
    	end
        local wanconfIPv6 = db.getRowWhere ("ipAddressTable","LogicalIfName='" .. LogIfName .. "' and addressFamily=10")
		local wanipIPv6 = "0000::00"
		if (wanconfIPv6 ~= nil) then
		    wanipIPv6 = wanconfIPv6["ipAddressTable.ipAddress"] or ''
		end
    	local protalURL = "https://" .. wanip .. "/portal/" .. row["SSLVPNPortalLayout.PortalLayoutName"]
        local protalURLIPv6 = "https://[" .. wanipIPv6 .. "]/portal/" .. row["SSLVPNPortalLayout.PortalLayoutName"]
    	local protalURLTitle = ""
    	local sslPortalName = ""
    	local displayUrl = protalURL
        local displayUrlIPv6 = protalURLIPv6
		protalURLTitleIPv6 = ""
    	local len = #portalName
    	if(len > 32) then
   			sslPortalName = row["SSLVPNPortalLayout.PortalLayoutName"]						    	
   			protalURLTitle = "https://" .. wanip .. "/portal/" .. row["SSLVPNPortalLayout.PortalLayoutName"]
    		portalName = string.sub(portalName,1,16) .. "..."
    		displayUrl = "https://" .. wanip .. "/portal/" .. portalName
            protalURLTitleIPv6 = "https://[" .. wanipIPv6 .. "]/portal/" .. row["SSLVPNPortalLayout.PortalLayoutName"]
            displayUrlIPv6 = "https://[" .. wanipIPv6 .. "]/portal/" .. portalName
    	end
		resTab.insertField (resultTab,"Layout Name",portalName)
        resTab.insertField (resultTab,"Description",bannerMessage)
        resTab.insertField (resultTab,"Use Count",row["SSLVPNPortalLayout.UseCount"])
		resTab.insertField (resultTab,"Portal URL (IPv4)",displayUrl)
        resTab.insertField (resultTab,"Portal URL (IPV6)",displayUrlIPv6)
	end
	printLabel("List of Layouts")
	resTab.print (resultTab, 0)
end

function sslvpnAppconfigShow(args)
	local table = db.getTable("SSLVPNPortForwardingAppsMap")
	local resultTab = {}
	local i = 0						

	for k,v in pairs(table) do
		i = i + 1							
		resTab.insertField (resultTab,"Row Id", v["SSLVPNPortForwardingAppsMap._ROWID_"])
		resTab.insertField (resultTab,"Server IP", (v["SSLVPNPortForwardingAppsMap.ServerIP"] or ""))
		resTab.insertField (resultTab,"Port", (v["SSLVPNPortForwardingAppsMap.Port"] or ""))
	end
	printLabel("Port Forwarding Application Configuration")
	resTab.print (resultTab, 0)
end

function sslvpnHostconfigShow(args)
	local table = db.getTable("SSLVPNPortForwardingFQDNMap")
	local resultTab = {}
	local i = 0						

	for k,v in pairs(table) do
		i = i + 1							
		resTab.insertField (resultTab,"Row Id", v["SSLVPNPortForwardingFQDNMap._ROWID_"])
		resTab.insertField (resultTab,"Server IP", (v["SSLVPNPortForwardingFQDNMap.ServerIP"] or ""))
		resTab.insertField (resultTab,"FQDN Name", (v["SSLVPNPortForwardingFQDNMap.FQDNName"] or ""))
	end
	printLabel("Port Forwarding Host Configuration")
	resTab.print (resultTab, 0)
end

function sslvpnResourceShow(args)
	local table = db.getTable("SSLVPNResource")
	local resultTab = {}
	local i = 0						

	for k,v in pairs(table) do
		i = i + 1							
		resTab.insertField (resultTab,"Row Id", v["SSLVPNResource._ROWID_"])
		resTab.insertField (resultTab,"Resource Name", (v["SSLVPNResource.ResourceName"] or ""))
        local service = ""
        if(v["SSLVPNResource.ServiceType"]=="VIRTUAL-PASSAGE") then
			service = "VPN Tunnel"
		elseif(v["SSLVPNResource.ServiceType"]=="VIRTUAL-TRANSPORT") then
			service = "Port Forwarding"
		elseif(v["SSLVPNResource.ServiceType"]=="all") then
		    service = "All"
		end
		resTab.insertField (resultTab,"Service", service)
		--resTab.insertField (resultTab,"Use Count", (v["SSLVPNResource.UseCount"] or ""))
	end
	printLabel("RESOURCES")
	resTab.print (resultTab, 0)
end

function sslvpnResourceObjectShow(args)
	local query = string.format([[
                                 SELECT *, _ROWID_ AS _ROWID_ FROM SSLVPNResourceObject WHERE ResourceName = '%s'
	                          ]], args[1])
	local table = db.getTable("SSLVPNResourceObject", true, query)
	local resultTab = {}
	local i = 0						

	for k,v in pairs(table) do
		i = i + 1							
		local ot = v["SSLVPNResourceObject.ObjectType"] or ""
		if (ot == "1") then
			ot = "IP Address"
		elseif (ot == "2") then
			ot = "IP Network"
		else
			ot = "UNKNOWN"
		end

		resTab.insertField (resultTab,"Row Id", v["SSLVPNResourceObject._ROWID_"])
		resTab.insertField (resultTab,"Object Type", ot)
        if (v["SSLVPNResourceObject.AddressType"] == "1") then
		    resTab.insertField (resultTab,"Object Address", (v["SSLVPNResourceObject.ObjectAddress"] or ""))
        elseif (v["SSLVPNResourceObject.AddressType"] == "2") then
		    resTab.insertField (resultTab,"Object Address", (v["SSLVPNResourceObject.ObjectAddress6"] or ""))
        end
		resTab.insertField (resultTab,"Mask Length", (v["SSLVPNResourceObject.MaskLength"] or ""))
		resTab.insertField (resultTab,"Start Port", (v["SSLVPNResourceObject.StartPort"] or ""))
		resTab.insertField (resultTab,"End Port", (v["SSLVPNResourceObject.EndPort"] or ""))
	end
	printLabel("RESOURCE OBJECTS")
	resTab.print (resultTab, 0)
end

function usersDomainsCfgInit(args)
	local configRow = {}
	if (args[1] ~= nil) then
		configRow = db.getRow("USERDBDomains", "USERDBDomains._ROWID_", args[1])
		if (configRow == nil)then
			print("Row does not exist\n")
			return -1,{}
		else
			return 1,configRow
		end
	else
        configRow["USERDBDomains._ROWID_"] = "-1"
		return 1,configRow
	end
end
function usersDomainsCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="USERDBDomains"
    configRow["USERDBDomains.TimeOut"] = "10"
    configRow["USERDBDomains.Retries"] = "3"
	RowId=configRow["USERDBDomains._ROWID_"]
	if (RowId == "-1") then
		configRow["USERDBDomains.DefaultDomain"]= "0"
		errorFlag, statusCode = sslAdmin.domainConfig(configRow, "-1", "add")
	--if edit
	else
        if (configRow["USERDBDomains.DefaultDomain"] == "1") then
		printCLIError("Default domain cannot be modified")
        return "ERROR", statusMessage
        end
        local domainRow = db.getRow("USERDBDomains","_ROWID_",RowId)
        if(domainRow["USERDBDomains.DomainName"] ~= configRow["USERDBDomains.DomainName"]) then
            configRow["USERDBDomains.DomainName"] = domainRow["USERDBDomains.DomainName"]
            print("Domain Name cannot be modified")
        end
        if(domainRow["USERDBDomains.AuthenticationType"] ~= configRow["USERDBDomains.AuthenticationType"]) then
            configRow["USERDBDomains.AuthenticationType"] = domainRow["USERDBDomains.AuthenticationType"]
            print("Authentication Type cannot be modified")
        end
		errorFlag, statusCode = sslAdmin.domainConfig(configRow, RowId, "edit")
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
 return errorFlag, statusMessage
end
function usersDomainsCfgInputVal(configRow)
	if (configRow["USERDBDomains.DomainName"] == nil or configRow["USERDBDomains.DomainName"] == "")then
		printCLIError("Invalid domain name")
		return false
	end
    if (string.len(configRow["USERDBDomains.DomainName"]) > 32)then
		printCLIError("Length of Domain name should not exceed 32 characters")
		return false
	end
    if(db.existsRowWhere("USERDBDomains","DomainName='"..configRow["USERDBDomains.DomainName"].."' AND _ROWID_!="..configRow["USERDBDomains._ROWID_"])) then
        printCLIError("Invalid domain name,domain name should be unique")
		return false
	end
	if (configRow["USERDBDomains.PortalLayoutName"] == nil or configRow["USERDBDomains.PortalLayoutName"] == "")then
		printCLIError("Invalid portal selected")
		return false
	end
	local testRow = db.getRow("SSLVPNPortalLayout","PortalLayoutName",configRow["USERDBDomains.PortalLayoutName"])
	if (testRow == nil or testRow == {})then
		printCLIError("Invalid portal selected")
		return false
	end 
	if (configRow["USERDBDomains.AuthenticationType"] == nil  or configRow["USERDBDomains.AuthenticationType"] == "")then
		printCLIError("Invalid authentication type selected")
		return false
	end 
	--if (configRow["USERDBDomains.TimeOut"] == nil  or configRow["USERDBDomains.TimeOut"] == "")then
		--printCLIError("Invalid timeout value")
		--return false
	--end 
	--if (configRow["USERDBDomains.Retries"] == nil  or configRow["USERDBDomains.Retries"] == "")then
		--printCLIError("Invalid retries value")
		--return false
	--end 
	if (configRow["USERDBDomains.AuthenticationType"] ~= "local")then
		if (configRow["USERDBDomains.AuthenticationServer"] == nil or configRow["USERDBDomains.AuthenticationServer"] == "")then
			printCLIError("Invalid authentication server")
			return false
		end	
	end
	if (configRow["USERDBDomains.AuthenticationType"] ~= "local" and configRow["USERDBDomains.AuthenticationType"] ~= "ntdomain" and  configRow["USERDBDomains.AuthenticationType"] ~= "active-directory" and configRow["USERDBDomains.AuthenticationType"] ~= "ldap")then
	
		if (configRow["USERDBDomains.AuthenticationRadiusSecret"] == nil or configRow["USERDBDomains.AuthenticationRadiusSecret"] == "")then
		printCLIError("Invalid authentication secret key")
		return false
		end
	end
	if (configRow["USERDBDomains.AuthenticationType"] == "ntdomain")then
		if (configRow["USERDBDomains.NTDomainWorkGroup"] == nil  or configRow["USERDBDomains.NTDomainWorkGroup"] == "")then
			printCLIError("Invalid workgroup")
			return false
		end
	end
	if (configRow["USERDBDomains.AuthenticationType"] == "active-directory")then
		if (configRow["USERDBDomains.ActiveDirectoryDomain"] == nil  or configRow["USERDBDomains.ActiveDirectoryDomain"] == "")then
			printCLIError("Invalid active directory domain")
			return false
		end
	end
	if (configRow["USERDBDomains.AuthenticationType"] == "ldap")then
		if (configRow["USERDBDomains.LDAPBaseDN"] == nil  or configRow["USERDBDomains.LDAPBaseDN"] == "")then
			printCLIError("Invalid ldap base dn")
			return false
		end
	end
	return true	
end

function usersDomainsCfgDel(args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
 	configRow=db.getRow("USERDBDomains","USERDBDomains._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	elseif (configRow["USERDBDomains.DefaultDomain"] == "1") then
		printCLIError("Default domains cannot be deleted")
		return "ERROR","Row cannot be deleted"
    else
		table.insert (idrow, configRow["USERDBDomains._ROWID_"])
		errorFlag, statusCode = sslAdmin.deleteDomains (idrow)
		if (errorFlag == "OK") then db.save() end
	end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
    return errorFlag, statusMessage	
end

function usersGroupsCfgInit(args)
	local configRow = {}
	if (args[1] ~= nil) then
		configRow = db.getRow("USERDBGroups", "USERDBGroups._ROWID_", args[1])
		if (configRow == nil)then
			print("Row does not exist\n")
			return -1,{}
		else
			return 1,configRow
		end
	else
        configRow["USERDBGroups._ROWID_"] = "-1"
		return 1,configRow
	end
end

function usersGroupsCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="USERDBGroups"
	RowId=configRow["USERDBGroups._ROWID_"]
	if (RowId ==  "-1") then
		configRow["USERDBGroups.DefaultGroup"]= "0"
		errorFlag, statusCode = sslAdmin.groupConfig(configRow, "-1", "add")
	--if edit
	else
        if (configRow["USERDBGroups.DefaultGroup"] == "1") then
		printCLIError("Default group cannot be modified")
        return "ERROR", statusMessage
        end
		errorFlag, statusCode = sslAdmin.groupConfig(configRow, RowId, "edit")
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
 return errorFlag, statusMessage
end
function usersGroupsCfgInputVal(configRow)
	
	if (configRow["USERDBGroups.GroupName"] == nil or configRow["USERDBDomains.GroupName"] == "")then
		printCLIError("Invalid group name")
		return false
	end
    if (string.len(configRow["USERDBGroups.GroupName"]) > 32)then
		printCLIError("Length of Group name should not exceed 32 characters")
		return false
	end
    if(db.existsRowWhere("USERDBGroups","GroupName='"..configRow["USERDBGroups.GroupName"].."' AND _ROWID_!="..configRow["USERDBGroups._ROWID_"])) then
        printCLIError("Invalid Group name,Group name should be unique")
		return false
	end
	if (configRow["USERDBGroups.DomainName"] == nil or configRow["USERDBGroups.DomainName"] == "")then
		printCLIError("Invalid domain name")
		return false
	end
	local testRow = db.getRow("USERDBDomains","DomainName",configRow["USERDBGroups.DomainName"])
	if (testRow == nil or testRow == {})then
		printCLIError("Invalid domain selected")
		return false
	end 
	if (configRow["USERDBGroups.GroupTimeOut"] == nil  or configRow["USERDBGroups.GroupTimeOut"] == "")then
		printCLIError("Invalid idle timeout value")
		return false
	end 
	return true
end
function usersGroupsCfgDel(args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
 	configRow=db.getRow("USERDBGroups","USERDBGroups._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	elseif (configRow["USERDBGroups.DefaultGroup"] == "1") then
		printCLIError("Default groups cannot be deleted")
		return "ERROR","Row cannot be deleted"
    	else
			-- Don't delete Group if it has users
		local groupName = db.getAttribute("USERDBGroups", "_ROWID_", args[1], "GroupName")
		if (db.existsRow ("USERDBUsers", "GroupName", groupName)) then
			printCLIError("Group containing Users can't be deleted !!")
			return false
		end
		table.insert (idrow, configRow["USERDBGroups._ROWID_"])
		errorFlag, statusCode = sslAdmin.deleteGroups (idrow)
		if (errorFlag == "OK") then db.save() end
	end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError(statusMessage)
    return errorFlag, statusMessage	
end
function usersCfgInit(args)
	local configRow = {}
	if (args[1] ~= nil) then
		configRow = db.getRow("USERDBUsers", "USERDBUsers._ROWID_", args[1])
		if (configRow == nil)then
			print("Row does not exist\n")
			return -1,{}
		else
			configRow["USERDBUsers.PasswordConfirm"]=configRow["USERDBUsers.Password"]
			return 1,configRow
		end
	else
        configRow["USERDBUsers._ROWID_"] = "-1"
		return 1,configRow
	end
end
function usersCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="USERDBUsers"
	local uname = ""
	local oldPwd = ""
	local newPwd = ""
	local newUname = ""
	local loggedinUser = ""
	RowId=configRow["USERDBUsers._ROWID_"]
	
    --if add
	if (RowId == "-1") then
		if (configRow["USERDBUsers.UserType"] == '7' or configRow["USERDBUsers.UserType"] == '5' or configRow["USERDBUsers.UserType"] == '8' ) then --l2tp case 
			configRow["USERDBUsers.GroupName"] = ''
			configRow["USERDBUsers.UserTimeOut"] = '10'
		end
		errorFlag, statusCode = sslAdmin.userConfig(configRow, "-1", "add")
	--if edit
	else
        	oldUSERDBUsersRow = db.getRow ("USERDBUsers", "_ROWID_", RowId) 
		if( oldUSERDBUsersRow == nil ) then 
			printCLIError("Configuration Failed!!") 
			return false 
		else
			if (oldUSERDBUsersRow["USERDBUsers.UserType"] == "0") then
				userType = "SSL VPN User"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "1") then
				userType = "SSLVPN Externally Authenticated User"		
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "3") then
				userType = "Administrator"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "4") then
                                userType = "Guest"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "5") then
                                userType = "IPSEC VPN User"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "7") then
                                userType = "L2TP User"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "8") then
                                userType = "PPTP User"
			elseif (oldUSERDBUsersRow["USERDBUsers.UserType"] == "9") then
				userType = "Local User"
			end
			if(oldUSERDBUsersRow["USERDBUsers.UserType"] == "7" or oldUSERDBUsersRow["USERDBUsers.UserType"] == "8")then --l2tp user
				if(configRow["USERDBUsers.UserType"] ~= oldUSERDBUsersRow["USERDBUsers.UserType"] ) then
					printCLIError(oldUSERDBUsersRow["USERDBUsers.UserName"] .. "'s UserType can not be modified .. using Default/Old userType :" .. userType)
            		configRow["USERDBUsers.UserType"] = oldUSERDBUsersRow["USERDBUsers.UserType"]
				end
			end
			if(configRow["USERDBUsers.UserName"] ~= oldUSERDBUsersRow["USERDBUsers.UserName"] ) then
				printCLIError(oldUSERDBUsersRow["USERDBUsers.UserName"] .. "'s Name can not be modified.. using Default/Old UserName : " .. oldUSERDBUsersRow["USERDBUsers.UserName"])
            			configRow["USERDBUsers.UserName"] = oldUSERDBUsersRow["USERDBUsers.UserName"]
			end
			if(configRow["USERDBUsers.GroupName"] ~= oldUSERDBUsersRow["USERDBUsers.GroupName"] ) then
				printCLIError(oldUSERDBUsersRow["USERDBUsers.UserName"] .. "'s GroupName can not be modified using Default/old GroupName : " .. oldUSERDBUsersRow["USERDBUsers.GroupName"])
            			configRow["USERDBUsers.GroupName"] = oldUSERDBUsersRow["USERDBUsers.GroupName"]
			end
			if(configRow["USERDBUsers.UserType"] ~= oldUSERDBUsersRow["USERDBUsers.UserType"] ) then
            	if(configRow["USERDBUsers.UserType"] == "7")then --if  changed to l2tp user
					printCLIError("User type can't be changed to l2tp user.")
					return false
            	end
                	if(configRow["USERDBUsers.UserType"] == "8")then --if  changed to pptp user
					printCLIError("User type can't be changed to PPTP user.")
					return false
            	end

			end
		end
		uname = db.getAttribute ("USERDBUsers", "_ROWID_", RowId, "UserName") or ''
		oldPwd = db.getAttribute ("USERDBUsers", "_ROWID_", RowId, "Password") or ''
		errorFlag, statusCode = sslAdmin.userConfig(configRow, RowId, "edit")
		newPwd = db.getAttribute ("USERDBUsers", "_ROWID_", RowId, "Password") or ''
		newUname = db.getAttribute ("USERDBUsers", "_ROWID_", RowId, "UserName") or ''
        	if (oldPwd ~= newPwd) then
	             db.setAttribute("SSLVPNActiveUsers", "UserName", uname, "SessionStatus", "EXPIRED_PWD_CHANGED")
        	end
        end
	-- save db if no error
	if (errorFlag == "OK") then
		db.save()
	end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    printCLIError(statusMessage)
	return errorFlag, statusMessage
    
end
function usersCfgInputVal(configRow)

	if (configRow["USERDBUsers.UserName"] == nil or configRow["USERDBUsers.UserName"] == "")then
		printCLIError("Enter valid username")
		return false
	end
    if (string.len(configRow["USERDBUsers.UserName"]) > 32)then
		printCLIError("Length of User name should not exceed 32 characters")
		return false
	end
    if(db.existsRowWhere("USERDBUsers","UserName='"..configRow["USERDBUsers.UserName"].."' AND _ROWID_!="..configRow["USERDBUsers._ROWID_"])) then
        printCLIError("Invalid User name,User name should be unique")
		return false
	end
--	if (configRow["USERDBUsers.FirstName"] == nil or configRow["USERDBUsers.FirstName"] == "")then
--		printCLIError("Enter valid firstname")
--		return false
--	end
--	if (configRow["USERDBUsers.LastName"] == nil or configRow["USERDBUsers.LastName"] == "")then
--		printCLIError("Enter valid lastname")
--		return false
--	end
	if (configRow["USERDBUsers.UserType"] == nil or configRow["USERDBUsers.UserType"] == "")then
		printCLIError("Enter valid usertype")
		return false
	end
	if (configRow["USERDBUsers.UserType"] ~= '7' and configRow["USERDBUsers.UserType"] ~= '5' and configRow["USERDBUsers.UserType"] ~= '8') then
		if (configRow["USERDBUsers.GroupName"] == nil or configRow["USERDBUsers.GroupName"] == "")then
			printCLIError("Enter valid groupname")
			return false
		end
		local testRow = db.getRow("USERDBGroups","GroupName",configRow["USERDBUsers.GroupName"])
		if (testRow == nil or testRow == {})then
			printCLIError("Enter existent groupname")
			return false
		end
		if (configRow["USERDBUsers.UserTimeOut"] == nil or configRow["USERDBUsers.UserTimeOut"] == "")then
			printCLIError("Enter valid idle timeout value")
			return false
		end
	    local domain = db.getAttribute("USERDBGroups", "GroupName", configRow["USERDBUsers.GroupName"], "DomainName")
	    local authType = db.getAttribute("USERDBDomains", "DomainName", domain, "AuthenticationType")
        if (authType == "local") then
	        if (configRow["USERDBUsers.Password"] == nil or configRow["USERDBUsers.Password"] == "")then
		        printCLIError("Enter valid password")
		        return false
	        end
	        if (configRow["USERDBUsers.PasswordConfirm"] == nil or configRow["USERDBUsers.PasswordConfirm"] == "")then
		        printCLIError("Please confirm password")
		        return false
	        end
	        if (configRow["USERDBUsers.Password"] ~= configRow["USERDBUsers.PasswordConfirm"])then
		        printCLIError("passwords didn't match")
		        return false
	        end
        end
    else
        if (configRow["USERDBUsers.Password"] == nil or configRow["USERDBUsers.Password"] == "")then
		    printCLIError("Enter valid password")
		    return false
	    end
	    if (configRow["USERDBUsers.PasswordConfirm"] == nil or configRow["USERDBUsers.PasswordConfirm"] == "")then
		    printCLIError("Please confirm password")
		    return false
	    end
	    if (configRow["USERDBUsers.Password"] ~= configRow["USERDBUsers.PasswordConfirm"])then
		    printCLIError("passwords didn't match")
		    return false
	    end
    end
	configRow["USERDBUsers.PasswordConfirm"] = nil
	return true
end
function usersDel(args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
 	configRow=db.getRow("USERDBUsers","USERDBUsers._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	elseif (configRow["USERDBUsers.DefaultUser"] == "1") then
		printCLIError("Default users cannot be deleted")
		return "ERROR","Row cannot be deleted"
    else
		table.insert (idrow, configRow["USERDBUsers._ROWID_"])
		errorFlag, statusCode = sslAdmin.deleteUsers(idrow)
		if (errorFlag == "OK") then db.save() end
	end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage	
end
function usersPoliciesCfgInit(args)
	configRow = db.getRow("USERDBUsers","_ROWID_", args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return -1,{}
	else
		return 1,configRow
	end
end
function usersPoliciesCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable="USERDBUsers"
	if(configRow == {} or configRow == nil) then
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, "-1", "add")
	else
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, configRow["USERDBUsers._ROWID_"], "edit")
	end
	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)	
	return errorFlag, statusMessage
end
function usersBrowserPoliciesCfgInit(args)
	configRow = db.getRow("USERDBUsers","_ROWID_", args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return -1,{}
	else
		return 1,configRow
	end
end
function usersBrowserPoliciesCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable="USERDBUsers"
	if(configRow["USERDBUsers.DelBrowserAndSave"] ~= nil and configRow["USERDBUsers.DelBrowserAndSave"] ~= "")then
		configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"] = configRow["USERDBUsers.DelBrowserAndSave"]
		configRow["USERDBUsers.DelBrowser"] = "1"
	end
	if(configRow["USERDBUsers.AddBrowserAndSave"] ~= nil and configRow["USERDBUsers.AddBrowserAndSave"] ~= "")then
		configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"] = configRow["USERDBUsers.AddBrowserAndSave"]
		configRow["USERDBUsers.AddBrowser"] = "1"
	end	
	if(configRow["USERDBUsers.AddBrowser"] == "1" and configRow["USERDBUsers.DelBrowser"] == "1")then
		configRow["USERDBUsers.AddBrowser"] = "0"
		configRow["USERDBUsers.DelBrowser"] = "0"
		configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"]=""
		printCLIError("Addition and deletion of browser(s) can't be performed simultaneously.")
	end
	if(configRow == nil or configRow == {}) then
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, "-1", "add")
	else
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, configRow["USERDBUsers._ROWID_"], "edit")
	end
	
	-- save db if no error
	if (errorFlag == "OK") then 
		db.save()
		DeleteStaleInput()
 	end
	if (configRow["USERDBUsers.AddBrowser"] == "1")then
		DBTable="SSLVPNUserLoginPolicyDefinedBrowser"
			configRow["SSLVPNUserLoginPolicyDefinedBrowser.UserName"] = configRow["USERDBUsers.UserName"]
			errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, "-1", "add")
		if (errorFlag == "OK") then db.save() end
	end
	if (configRow["USERDBUsers.DelBrowser"] == "1")then
		DBTable="SSLVPNUserLoginPolicyDefinedBrowser"
		local testRow = db.getRow("SSLVPNUserLoginPolicyDefinedBrowser","UserName='"..configRow["USERDBUsers.UserName"].."' and ClientBrowserName",configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"])
		if (testRow == nil or testRow == {})then
			printCLIError("No such Browser  added in the list")
			return "ERROR","No such Browser added in the list"
		else
			local rows={}
			rows["SSLVPNUserLoginPolicyDefinedBrowser._ROWID_"]=testRow["SSLVPNUserLoginPolicyDefinedBrowser._ROWID_"]
			errorFlag, statusCode = sslAdmin.loginPolicyConfig (rows,"-1","delete")
		end
		if (errorFlag == "OK") then db.save() end
	end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	
	return errorFlag, statusMessage
end


function usersBrowserPoliciesCfgVal(configRow)
	if (configRow["USERDBUsers.LoginFromBrowser"] == nil ) then
		printCLIError("Invalid Configuration/Setting.(Enable_or_Disable_login_from_defined_browsers)")
		return false
	end
	if(configRow["USERDBUsers.UserName"] == nil or configRow["USERDBUsers.UserName"] == "") then
		printCLIError("Error in Browser Policies configuration(<<invalid ROW index>>).")
		return false
	end
	if (configRow["USERDBUsers.DelBrowser"] == "1" and configRow["USERDBUsers.AddBrowser"] == "1")then
		printCLIError("Cannot perform Addition and deletion of client browser simultanesouly")
		return false
	end
	if (configRow["USERDBUsers.DelBrowser"] == "1" or configRow["USERDBUsers.AddBrowser"] == "1")then
		if (configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"] == nil or configRow["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"] == "")then
		printCLIError("select valid client browser to add/delete")
		return false
		end
	end
	return true
end

function usersIpPoliciesCfgInit(args)
	configRow = db.getRow("USERDBUsers","_ROWID_", args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return -1,{}
	else
        if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "10") then
            configRow["SSLVPNUserLoginPolicyDefinedAddress.Address6"] = configRow["SSLVPNUserLoginPolicyDefinedAddress.Address"]
            configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"]= configRow["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"]
        end
		return 1,configRow
	end
end

function usersIpPoliciesCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable="USERDBUsers"
    if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "10") then
        configRow["SSLVPNUserLoginPolicyDefinedAddress.Address"] = configRow["SSLVPNUserLoginPolicyDefinedAddress.Address6"]
        configRow["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"]= configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"]
    end
    configRow["SSLVPNUserLoginPolicyDefinedAddress.Address6"] = nil
    configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"] = nil
	if(configRow == nil or configRow == {}) then
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, "-1", "add")
	else
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, configRow["USERDBUsers._ROWID_"], "edit")
	end
    	if (configRow["USERDBUsers.AddIp"] == "1")then
		DBTable="SSLVPNUserLoginPolicyDefinedAddress"
		configRow["SSLVPNUserLoginPolicyDefinedAddress.UserName"] = configRow["USERDBUsers.UserName"]
			errorFlag, statusCode = sslAdmin.loginPolicyConfig (configRow, "-1", "add")
		if (errorFlag == "OK") then db.save() end
    	end	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	return errorFlag, statusMessage
end

function usersIpPoliciesCfgVal(configRow)
	if (configRow["USERDBUsers.AddIp"] == "1")then
        if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "")then
			printCLIError("Enter ip_version")
			return false
		end
        if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "10" and db.getAttribute("networkInfo","_ROWID_","1","netWorkMode") ~= "3") then
            printCLIError("Please Set IP Mode to IPv4/IPv6 to configure ip_version option as IPv6.")
            return false
        end
		if (configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == "")then
			printCLIError("Enter valid address type")
			return false
		end
        if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "2") then
		    if (configRow["SSLVPNUserLoginPolicyDefinedAddress.Address"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.Address"] == "" or configRow["SSLVPNUserLoginPolicyDefinedAddress.Address"] == "0.0.0.0")then
			    printCLIError("Enter valid  IPv4 address")
			    return false
		    end
        elseif (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "10") then
            if (configRow["SSLVPNUserLoginPolicyDefinedAddress.Address6"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.Address6"] == "" ) then
                printCLIError("Enter valid  IPv6 address")
			    return false
		    end
        end
        if (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "2") then
		    if (configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == "1" and (configRow["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"] == ""))then
			    printCLIError("Enter valid mask length")
			    return false
		    elseif (configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == "0")then
		        configRow["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"]="32"
		    end
        elseif (configRow["SSLVPNUserLoginPolicyDefinedAddress.IPVersion"] == "10") then
            if (configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == "1" and (configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"] == nil or configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"] == ""))then
			    printCLIError("Enter valid Prefix length")
			    return false
		    elseif (configRow["SSLVPNUserLoginPolicyDefinedAddress.AddressType"] == "0")then
		        configRow["SSLVPNUserLoginPolicyDefinedAddress.PrefixLength"]="128"
		    end
        end
	end	
	return true
end
function usersIpPolicyDel(args)

    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
	DBTable="SSLVPNUserLoginPolicyDefinedAddress"
 	configRow=db.getRow("SSLVPNUserLoginPolicyDefinedAddress","SSLVPNUserLoginPolicyDefinedAddress._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	else
		idrow["SSLVPNUserLoginPolicyDefinedAddress._ROWID_"]=configRow["SSLVPNUserLoginPolicyDefinedAddress._ROWID_"]
		errorFlag, statusCode = sslAdmin.loginPolicyConfig (idrow, "-1", "delete")
		if (errorFlag == "OK") then db.save() end
	end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage	
end
function usersDomainsGet()

	local resultTab = {}
    local resultTab1 = {}
    local status = ""
    local localAuth = db.getRow ("SSLVPNUserLoginLocalDbStatus","_ROWID_","1")
    if(localAuth["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] == "0") then
        status = "No"
    elseif(localAuth["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] == "1") then
        status = "Yes"
    end
    resTab.insertField (resultTab1,"Local Authentication Disable",status)
	local table = db.getTable ("USERDBDomains")
	local i = 0						
		for k,v in pairs(table) do
			i = i + 1							
			local row = table[i]
			local domainname=row["USERDBDomains.DomainName"]
			if (row["USERDBDomains.DefaultDomain"] == '1') then
				domainname=domainname.."*"
			end
			resTab.insertField (resultTab,"Row_Id",row["USERDBDomains._ROWID_"])
			resTab.insertField (resultTab,"Domain Name",domainname)
			local authType = row["USERDBDomains.AuthenticationType"]
				if (authType == "local") then
					authType = "Local User Database"
				elseif (authType == "radius_pap") then
					authType = "Radius-PAP"
				elseif (authType == "radius_chap") then
					authType = "Radius-CHAP"
				elseif (authType == "radius_mschap") then
					authType = "Radius-MSCHAP"
				elseif (authType == "radius_mschapv2") then
					authType = "Radius-MSCHAPv2"
				elseif (authType == "ntdomain") then
					authType = "NT Domain"
				elseif (authType == "active-directory") then
					authType = "Active Directory"
				elseif (authType == "ldap") then
					authType = "LDAP"
				end
			resTab.insertField (resultTab,"Autentication Type",authType)
			resTab.insertField (resultTab,"Portal Layout Name",row["USERDBDomains.PortalLayoutName"] )
		 end
        printLabel("Local Authentication")
        resTab.print (resultTab1, 0)
		printLabel("List of Domains")
		resTab.print (resultTab, 0)
end
function usersGroupsGet()
		local resultTab = {}
		local mark = ""
		local table = db.getTable ("USERDBGroups")
		local i = 0
		for k,v in pairs(table) do
			i = i + 1
			local row = table[i]
			mark = " "
			local groupName=""
			if (row["USERDBGroups.DefaultGroup"] == '1') then
				if(row["USERDBGroups.GroupName"] == "geardomain") then	
					mark = "*"
				end
			end
			   groupName=row["USERDBGroups.GroupName"]..mark
			   resTab.insertField (resultTab,"Row_Id",row["USERDBGroups._ROWID_"])
			   resTab.insertField (resultTab,"Name",groupName)
		    	   resTab.insertField (resultTab,"Domain",row["USERDBGroups.DomainName"])
		  end
		printLabel("List of Groups")
		resTab.print (resultTab, 0)
end
--function for showing active users 
function activeUsersGet()
	local resultTab = {}
	local i = 0
	local table = db.getTable("SSLVPNActiveUsers")
	for k,v in pairs(table) do
	    i = i + 1
	    local row = table[i]
	    resTab.insertField(resultTab,"UserName: ", row["SSLVPNActiveUsers.UserName"])
	    resTab.insertField(resultTab,"GroupName: ", row["SSLVPNActiveUsers.GroupName"])
	    resTab.insertField(resultTab,"LoginAddress: ",row["SSLVPNActiveUsers.LoginAddress"] or '')
	    resTab.insertField(resultTab,"LoginTime: ",row["SSLVPNActiveUsers.LoginTime"] or '')
	end
	resTab.print(resultTab,0)
	return true
end

function usersGet()
	local resultTab = {}
	local mark = ""
	local table = db.getTable ("USERDBUsers")
	local i = 0						
		for k,v in pairs(table) do
			i = i + 1							
			local row = table[i]
			mark = " "
			if (row["USERDBUsers.DefaultUser"] == '1') then
				mark ="*"
			end
            local userName=row["USERDBUsers.UserName"]..mark
			resTab.insertField (resultTab,"Row_Id",row["USERDBUsers._ROWID_"])
			resTab.insertField (resultTab,"User Name",userName)
            resTab.insertField (resultTab,"Group",row["USERDBUsers.GroupName"])
			local userType=""
			if (row["USERDBUsers.UserType"] == "0") then
				userType = "SSL VPN User"
			elseif (row["USERDBUsers.UserType"] == "1") then
				userType = "SSLVPN Externally Authenticated User"		
			elseif (row["USERDBUsers.UserType"] == "3") then
				userType = "Administrator"
			elseif (row["USERDBUsers.UserType"] == "4") then
                                userType = "Guest"
			elseif (row["USERDBUsers.UserType"] == "5") then
                                userType = "IPSEC VPN User"
			elseif (row["USERDBUsers.UserType"] == "7") then
                                userType = "L2TP User"
			elseif (row["USERDBUsers.UserType"] == "8") then
                                userType = "PPTP User"
			elseif (row["USERDBUsers.UserType"] == "9") then
				userType = "Local User"
			end
			resTab.insertField (resultTab,"Type",userType)
			if ( row["USERDBUsers.UserType"] ~= "7" and row["USERDBUsers.UserType"] ~= "5") then
				local group = row["USERDBUsers.GroupName"]
				local domain = db.getAttribute("USERDBGroups", "GroupName", group, "DomainName") or ''
                resTab.insertField (resultTab,"Authentication Domain",domain)
				--local authType = db.getAttribute("USERDBDomains", "DomainName", domain, "AuthenticationType")
				--if (authType == "local") then
					--authType = "Local User Database"
				--elseif (authType == "radius_pap") then
					--authType = "Radius-PAP"
				--elseif (authType == "radius_chap") then
					--authType = "Radius-CHAP"
				--elseif (authType == "radius_mschap") then
					--authType = "Radius-MSCHAP"
				--elseif (authType == "radius_mschapv2") then
					--authType = "Radius-MSCHAPv2"
				--elseif (authType == "ntdomain") then
					--authType = "NT Domain"
				--elseif (authType == "active-directory") then
					--authType = "Active Directory"
				--elseif (authType == "ldap") then
					--authType = "LDAP"
				--end
				--resTab.insertField (resultTab,"Authentication Domain",authType)
			else
				resTab.insertField (resultTab,"Authentication Domain","")
			end
			local loginStatus = ""
			if(row["USERDBUsers.DenyLogin"] == "1" and row["USERDBUsers.DenyLoginFromWan"] == "1") then
				loginStatus = "Disabled"
			elseif (row["USERDBUsers.DenyLogin"] == "1" and row["USERDBUsers.DenyLoginFromWan"] == "0") then
				loginStatus = "Enabled (WAN only)"
			elseif (row["USERDBUsers.DenyLogin"] == "0" and row["USERDBUsers.DenyLoginFromWan"] == "1") then
				loginStatus = "Enabled (LAN only)"
			elseif (row["USERDBUsers.DenyLogin"] == "0" and row["USERDBUsers.DenyLoginFromWan"] == "0") then
				loginStatus = "Enabled (LAN and WAN)"
			end
			resTab.insertField (resultTab,"Login Status",loginStatus)
	end
		printLabel("List of Users")
		resTab.print (resultTab, 0)

end
function usersLoginPolicyGet(args)

 	configRow=db.getRow("USERDBUsers","USERDBUsers._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	else
		printLabel("User Login Policies")
		print("User Name: "..configRow["USERDBUsers.UserName"])
		local disable_login=""
		local deny_login=""
		if (configRow["USERDBUsers.DenyLogin"] == "1")then
			disable_login="Yes"
		else
			disable_login="No"
		end
		if (configRow["USERDBUsers.DenyLoginFromWan"] == "1")then
			deny_login="Yes"
		else
			deny_login="No"
		end
		print("Disable Login: "..disable_login)
		print("Deny Login from Wan Interface: "..deny_login)
	end
end

function usersBrowserPolicyGet(args)

 	configRow=db.getRow("USERDBUsers","USERDBUsers._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	else
		printLabel("User Browser Policies")
		print("User Name: "..configRow["USERDBUsers.UserName"])
		local deny_login=""
		if (configRow["USERDBUsers.LoginFromBrowser"] == "1")then
			deny_login="Yes"
		else
			deny_login="No"
		end
		print("Allow Login from Defined Browser: "..deny_login)
		local testRow=db.getRows("SSLVPNUserLoginPolicyDefinedBrowser","UserName",configRow["USERDBUsers.UserName"])
		printLabel("Defined Browsers")
		if ( #testRow ~= 0)then
			local i = 0
			for k,v in pairs(testRow) do
				i = i + 1							
				local row = testRow[i]

				if(row["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"]=="MSIE") then
				print("Internet Explorer ")
				elseif(row["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"]=="Navigator") then
 				print("Netscape Navigator")
				else
				print(row["SSLVPNUserLoginPolicyDefinedBrowser.ClientBrowserName"])
				end
			end	
		end
	end
end
function usersIpPolicyGet(args)

	local resultTab = {}
 	configRow=db.getRow("USERDBUsers","USERDBUsers._ROWID_",args[1])
	if (configRow == nil)then
		printCLIError("Row does not exist")
		return "ERROR","Row does not exist"
	else
		printLabel("User Ip Policies")
		print("User Name: "..configRow["USERDBUsers.UserName"])
		local deny_login=""
		if (configRow["USERDBUsers.LoginFromIP"] == "1")then
			deny_login="Yes"
		else
			deny_login="No"
		end
		print("Allow Login from Defined Address: "..deny_login)
		printLabel("Ip Addresses")
		local uname = configRow["USERDBUsers.UserName"] or ''
		local type = ""
		local table = db.getRowsWhere ("SSLVPNUserLoginPolicyDefinedAddress","UserName='" ..  uname .. "'")
		local i = 0								
			for k,v in pairs(table) do
				i = i + 1
				local row = table[i]
				resTab.insertField (resultTab,"Row_Id",row["SSLVPNUserLoginPolicyDefinedAddress._ROWID_"])
				if(row["SSLVPNUserLoginPolicyDefinedAddress.AddressType"]== "0") then
					type = "IP Address"
				else
					type = "IP Network"
				end
				resTab.insertField (resultTab,"Source Address Type",type)
			        resTab.insertField (resultTab,"Network/IP Address",row["SSLVPNUserLoginPolicyDefinedAddress.Address"])
				resTab.insertField (resultTab,"Mask Length",row["SSLVPNUserLoginPolicyDefinedAddress.MaskLength"])
			end
		resTab.print (resultTab, 0)
	end

end


function usersDomainsLocalAuth(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    local configRow = db.getRow ("SSLVPNUserLoginLocalDbStatus","_ROWID_","1")
    configRow["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] = args[1]
    errorFlag, statusCode = sslAdmin.localAuthdomainConfig (configRow, "1", "edit")
    -- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    printCLIError (statusMessage)
	return errorFlag, statusMessage
end
