-- tr069MgmtCLI.lua
-- Abdul Shukur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 19feb11,Asr written.
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/tr069"

--
-- This routine validates the accessMgmt configuration information.
function tr069MgmtInputVal(configRow)
    if (configRow["tr069Config.enableTr069"] == "0" ) then
        return true
    end
    if (configRow["tr069Config.isStunEnable"] == "1" ) then
        if (configRow["tr069Config.stunIp"] == nil) then
            printCLIError ("Invalid stunIp address")
            return false
        end
    end
    if (configRow["tr069Config.isStunEnable"] == "1" ) then
        if (configRow["tr069Config.stunUserName"] == nil) then
            printCLIError ("Invalid stunUserName")
            return false
        end
    end
    if (configRow["tr069Config.isStunEnable"] == "1" ) then
        if (configRow["tr069Config.stunPassword"] == nil) then
            printCLIError ("Invalid stunPassword")
            return false
        end
    end
    return true
end

--
-- This routine reads in the current tr069 configuration information
-- into a configuration object.
function tr069MgmtInit(args)
    local rowId = -1

    local configRow = db.getRow("tr069Config", "_ROWID_", "1")
    if (configRow == nil) then
       return rowId, {}
    end
    
    rowId = configRow["tr069Config._ROWID_"]   
    return rowId, configRow
end

--
-- This routine implements the CLI command to show remote mgmt
-- configuration
function  tr069MgmtGet()
    local tr069MgmtTbl = nil
        
        printLabel ("TR069 Mgmt Configuration")    
        tr069MgmtTbl = db.getRow ("tr069Config","_ROWID_","1")
        if (tr069MgmtTbl ~= nil) then
            if (tr069MgmtTbl["tr069Config.enableTR069"] == "0") then
                print ("Remote access through TR069 is currently disabled")
            else
                    print ("Port1 used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.port1"])
                    print ("Port2 used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.port2"])
                    print ("acsUserName used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.acsUserName"])
                    print ("acsPassword used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.acsPassword"])
                    print ("ConReqUserName used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.ConReqUserName"])
                    print ("ConReqPassword used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.ConReqPassword"])
                    print ("acsIpAddr used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.acsIpAddr"])
                    if (tr069MgmtTbl["tr069Config.isStunEnable"] == "0") then
                        print ("STUN disabled")
                    else
                        print ("STUN status for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.isStunEnable"])
                        print ("stunUserName used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.stunUserName"])
                        print ("stunPassword used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.stunPassword"])
                        print ("stunIp used for TR069 Mgmt is - " .. tr069MgmtTbl["tr069Config.stunIp"])
                    end
            end
        end
end


--
-- This routine saves the modified remote management configuration information.
function tr069MgmtCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = ""

    local tr069 = db.getRow ("tr069Config", "_ROWID_", "1")
    
    if (tr069 == nil) then
        print("Add")
        errorFlag, statusMessage = tr069Config.config (configRow, "-1", "add")
    else
        print("Edit")
        errorFlag, statusMessage = tr069Config.config (configRow, "1", "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then 
        print("Saving...")
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
    return errorFlag, statusMessage
end


